/****************************************************************************
   Module Name          :  read_cfg.c
   Organization         :  IVRS (International) Ltd
   Edit History         :

2001.11.01	v1.00	Keith Chue	First Created
2002.12.17	v1.01	Silvia Wan	Enhance to CPP version
*****************************************************************************

   Remarks:

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SysUtil.h"
#include "ReadCfg.h"

#define MODULE_NAME	"READ_CFG"
#define VERSION_NUM	"1.01"
#define REVISION_DATE	"20021217"

static char version[]=MODULE_NAME" Version "VERSION_NUM" Copyright (c)  "REVISION_DATE;

#define DEBUG_LINE	printf(MODULE_NAME":DEBUG_LINE = %04d\n", __LINE__);


ReadCfg::ReadCfg()
{
	gp = NULL;
}


ReadCfg::~ReadCfg()
{
	delete gp;
}


int ReadCfg::init(ReadCfg_S *p)
{
	gp = p;
	return 0;
}


int ReadCfg::alloc_mem()
{
	void	*p = NULL;

	p = malloc(sizeof(ReadCfg_S));

	if (p != NULL) {
		gp = (ReadCfg_S *)p;
		memset(gp, 0, sizeof(ReadCfg_S));
		return 1;
	} else {
		return -1;
	}
}


int ReadCfg::read_param(char *filename, char *tag_int, char *tag_str)
{
	FILE		*fp;
	char		file_path[128];
	char		linebuf[1024];
	int		value_int;
	char		value_str[128];
	int		line;
	int		match_token = 0;
	char		token[128];
	ReadCfg_S	this_gp;

	if (getcfgpath(file_path, filename) == NULL) {
		printf("Can't get full path of %s\n", filename);
		return -1;
	}

	if ((fp=fopen(file_path,"r")) == NULL) {
		sprintf(linebuf, MODULE_NAME":Cannot open %s", file_path);
		puts(linebuf);
		return -1;          /*error exit*/
	}

	memset(&this_gp, 0, sizeof(this_gp));
	strcpy(token, tag_int);
	line = 0;
	match_token = 0;

	while (memset(linebuf, 0, sizeof(linebuf)) && fgets(linebuf, sizeof(linebuf)-1, fp)) {
		if (!linebuf[0] || linebuf[0]==';') {
			continue;
		}

		if (match_token == 0 && !strstr(linebuf, token)) {
			continue;
		} else 
		if (match_token == 0 && line == 0) {
			match_token = 1;
			continue;
		} else {
			/***  read config ****/
		}

		value_int = 0;
		memset(value_str, 0, sizeof(value_str));
		if (sscanf(linebuf, "%s", value_str) >= 1 && line < MAX_CFG_INT_PARAM) {
			if (!strncasecmp(value_str, "0x", 2)) {
				sscanf(value_str+2, "%x", &value_int);
			} else {
				sscanf(value_str, "%d", &value_int);
			}
			this_gp.i[line++] = value_int;	
		} else {
			break;
		}
	}
	this_gp.i_num = line;

	rewind(fp);

	strcpy(token, tag_str);
	line = 0;
	match_token = 0;
	while (memset(linebuf, 0, sizeof(linebuf)) && fgets(linebuf, sizeof(linebuf)-1, fp)) {
		if (!linebuf[0] || linebuf[0]==';') {
			continue;
		}

		if (match_token == 0 && !strstr(linebuf, token)) {
			continue;
		} else 
		if (match_token == 0 && line == 0) {
			match_token = 1;
			continue;
		} else {
			/***  read config ****/
		}

		memset(value_str, 0, sizeof(value_str));

		if (sscanf(linebuf, "%s", value_str) >= 1 && line < MAX_CFG_STR_PARAM) {
			strncpy(this_gp.s[line++], value_str, MAX_CFG_STR_LEN);
		} else {
			break;
		}
	}
	this_gp.s_num = line;

	fclose(fp);
	memcpy(gp, &this_gp, sizeof(this_gp));

	return 1;
}



void ReadCfg::view_param()
{
	int	i;

	for (i=0; i < gp->i_num; i++) {
		printf("INT %02d:\t%d\n", i, gp->i[i]);
	}

	for (i=0; i < gp->s_num && gp->s[i][0]; i++) {
		printf("STR %02d:\t%s\n", i, gp->s[i]);
	}

	return;
}


char *ReadCfg::get_s(int pos)
{
	if (gp && pos >= 0 && pos < gp->s_num && gp->s[pos][0]) {
		return gp->s[pos];
	} else {
		return NULL;
	}
}


int ReadCfg::get_i(int pos)
{
	if (gp && pos >=0 && pos < gp->i_num) {
		return gp->i[pos];
	} else {
		return 0;
	}
}
