#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>

#define MODULE_NAME	"SMClient"
#define VERSION_NUM	"1.20"
#define REVISION_DATE	"20060417"
static char version[]="\n" MODULE_NAME " Version " VERSION_NUM " Copyright (c) IVRS "REVISION_DATE "\n";

#include <IPC_Memory.h>
#include "GeneralErrorException.h"
#include "SMClient.h"
#include "LogUtil.h"
#include "SysUtil.h"

#include "util.h"
#include "shared.h"
#include "intm_msg.h"
#include "intm_msg_scme.h"

/*** USED FOR INTM_MESSAGE ***/
#define MESSAGE_ID           0xFD
#define REGISTERED_DELIVERY  0xFE
#define PROTOCOL_ID          0xFF
/***************************/

SMClient	*smclnt;
celcom_alarmstruct     *alarm_data;

extern "C" {
	void signal_hndlr(int sig_num);
}

#define DEBUG_LINE	printf("DEBUG_LINE = %04d\n", __LINE__);

int main(int argc, char **argv) 
{
	int	rc;

	try {
		smclnt = new SMClient();

		init_log((char *)"SMCLNT");
		loginfo(PROCESS_LOG, INFO, (char *)"SMClient starting...");

		smclnt->init(argc, argv);

		smclnt->set_signal();
		smclnt->waitEvent();

		smclnt->disconnectAllServers();
/// ForServer 	smclnt->disconnectAllClients();

		loginfo(PROCESS_LOG, INFO, (char *)("Program "MODULE_NAME" v"VERSION_NUM" ended"));

	} catch (GeneralErrorException err) {
		cout << err.getMessage() << '\n';
	}

	return 0;
}


SMClient::SMClient()
{
	int	i = 0;

	quit = 0;
	listen_port = 0;
	reload_flag = 0;
	debug_level = 0;
	
	memset(config_file, 0, sizeof(config_file));
	memset(bind_info, 0, sizeof(bind_info));
	memset(profile, 0, sizeof(profile));
	memset(client_map, 0, sizeof(client_mapping_struct)*MAX_CLIENT_MAPPING_DATA);

	smclntq   = NULL;
	readCfg   = NULL;
	smclntCfg = NULL;
	logFile   = NULL;
	uni_util  = NULL;
	for (i=0; i<MAX_APP_QUEUE; i++) {
		appq[i] = NULL;
	}
}


SMClient::~SMClient()
{
	int	i = 0;

	delete smclntq;
	delete readCfg;
	delete smclntCfg;
	delete logFile;
	delete uni_util;
	for (i=0; i<MAX_APP_QUEUE; i++) {
		delete appq[i];
	}
}

void signal_hndlr(int sig_num)
{
	printf("sig_num = %d\n", sig_num);
	switch (sig_num) {
	case SIGALRM:
	case SIGUSR2:
		signal(sig_num, signal_hndlr);
		break;

	case SIGHUP:
		smclnt->reload_flag = 1;
		signal(sig_num, signal_hndlr);
		break;

	case SIGUSR1:
		if (smclnt->debug_level) {
			smclnt->debug_level = 0;
		} else {
			/*smclnt->debug_level = 100;*/
			smclnt->debug_level = 111;
		}
		printf("DEBUG_LEVEL = %d\n", smclnt->debug_level);
		signal(sig_num, signal_hndlr);
		break;

	default:
		loginfo(PROCESS_LOG, INFO, (char *)"signal %d received, quit", sig_num);
		smclnt->quit = 1;
		break;
	}
	error_monitor2(alarm_data,ALM_SMCLNT_ERR,1,3600);
}


void SMClient::set_signal()
{
	signal(SIGINT, signal_hndlr);
	signal(SIGUSR1, signal_hndlr);
	signal(SIGUSR2, signal_hndlr);
	signal(SIGALRM, signal_hndlr);
	signal(SIGTERM, signal_hndlr);
	signal(SIGKILL, signal_hndlr);
	signal(SIGSEGV, signal_hndlr);
	signal(SIGBUS, signal_hndlr);
	signal(SIGHUP,SIG_IGN);
	signal(SIGCHLD, signal_hndlr);

	return;
}


void SMClient::init(int argc, char **argv)
{
	char	logname[128];
	char	bakname[128];

/****************************/
        SysMem = new IPC_Memory();
        SysMem->init(ALARM_STRUCT_KEY,PERMS);
        if (SysMem->is_valid() != 1)
        {
                throw GeneralErrorException((char *)"Attach Shared memory failed");
        }
        SysMem->get_SharedMemoryAddress((void **)&alarm_data);
/***************************/

	sprintf(config_file, "%s.dat", argv[0]);

	readServerInfoTagConfig(config_file);
// ForServer	readClientInfoTagConfig(config_file);

	readCfg = new ReadCfg();
	readCfg->alloc_mem();
	readCfg->read_param(config_file, (char *)"[SCME_INT]", (char *)"[SCME_STR]");
	readCfg->view_param();

	smclntCfg = new SMClientCfg();
	smclntCfg->ReadConfig(config_file, bind_info, profile, MAX_SERVER_NUM);

	readAppRouteConfig(config_file);

	sprintf(logname, "%s.log", argv[0]); 
	sprintf(bakname, "%s.lbk", argv[0]); 
	logFile = new LogFile(logname, bakname);
	logFile->setMaxLogSize(10000000);
	debug_level = readCfg->get_i(GI_DEBUG_LEVEL);

	if (readCfg->get_i(GI_MSGQ_FLAG) == 1) {
		// init message queue
		try {
			smclntq = new IPC_Queue();
			smclntq->init(readCfg->get_i(GI_MSGQ_KEY), IPC_CREAT|PERMS);
			for (int i=0; i<MAX_APP_QUEUE; i++) {
				if (mtype[i]) {
					appq[i] = new IPC_Queue();
					appq[i]->init(queue[i], IPC_CREAT|PERMS);
				}
			}
		} catch (GeneralErrorException err) {
			loginfo(PROCESS_LOG, MAJOR, (char *)"init queue failed");
			error_monitor2(alarm_data,ALM_MSGQ_ERR,1,3600);
			throw err;
        	}
	}

	uni_util = new UniCode_Util();

/******************  ForServer
	listen_port = readCfg->get_i(GI_SERVER_PORT);
	printf("listen_port %d\n",listen_port);

	if (listen_port > 1024 && listen_port < 65536) {
		// flag = 0, don't check client IP, 1 = check
		initServer(listen_port, 1);
	} else {
		listen_port = 0;
	}
********************/

	return;
}


int SMClient::readAppRouteConfig(char *filename)
{
	FILE	*fp;
	int	index, rc;
	char	buffer[256];
	char	fullpath[256];
	char	str1[40], str2[40], str3[40];
	int	value1, value2, value3;
	char	*tag_name = (char *)"[APP_ROUTE]";
	int	phase = 0;	// 0 = before tag, 1 = inside tag, 2 = after tag

	getcfgpath(fullpath, filename);

	if ((fp = fopen(fullpath, "r")) == NULL) {
		throw GeneralErrorException((char *)"cannot open config file %s", fullpath);
	}

	while (fgets(buffer, sizeof(buffer)-1, fp) && phase < 2) {
		if (buffer[0] == '[') {
			if (phase == 1) {
				phase = 2;
			} else if (!strncmp(buffer, tag_name, strlen(tag_name))) {
				phase = 1;
			}
		} else if (phase == 1) {
			memset(str1, 0, sizeof(str1));
			memset(str2, 0, sizeof(str2));
			memset(str3, 0, sizeof(str3));
			if (buffer[0] == ';' || buffer[0] == '#' || strlen(buffer) < 2) {
				// ignore
			} else if (sscanf(buffer, "%s %s %s", str1, str2, str3) >= 3) {
				value1 = str2long(str1);
				value2 = str2long(str2);
				value3 = str2long(str3);	// send mtype must be > 0
				if (value1 >= 0 && value1 < MAX_APP_QUEUE && value3) {
					queue[value1] = value2;
					mtype[value1] = value3;
				}
				printf("%02d	0x%04X	%d\n", value1, value2, value3);
			}
		}
	}

	fclose(fp);

	return 1;
}


void SMClient::waitEvent()
{
	int		i, num, rc;
	time_t		last_current_time, current_time;
	long		recv_mtype = 0;

	last_current_time = current_time = time(NULL);

	while (!quit) {

		current_time = time(NULL);

		if (last_current_time != current_time) {
			check_timeout(current_time);
			last_current_time = current_time;
		}

		// poll if there is message from server
		pollServerEvent(10);

		// poll if there is message from message queue
		unsigned char in_msg[65536];
		size_t in_msg_size = 0;
		recv_mtype = readCfg->get_i(GI_MSGQ_MTYPE);
		rc = smclntq->recv_nowait(in_msg, in_msg_size, recv_mtype);
		if (rc > 0) {
			processQueueMessage((intm_msg*)in_msg, in_msg_size);
		}
	}	// end of while (!quit)

	return;
}


void SMClient::check_timeout(time_t time_now)
{
	int			i;
	static time_t		last_elink = 0;
	SMPP_M_EnquireLink	smpp_e;
	unsigned char		uc_out_smpp[2000];
	int			elink_interval = 0;
	int			conn_interval = 0;


	elink_interval = readCfg->get_i(GI_ELINK_INTERVAL);
	conn_interval = readCfg->get_i(GI_CONN_INTERVAL);

	for(i=0; i<MAX_SERVER_NUM; i++) {
		switch(server_status[i]) {
		case CONN_IDLE:
			if (isValidServerConfiguration(i) && !isValidServerConnection(i) && ((time_now - server_et[i]) > conn_interval)) {
				loginfo(PROCESS_LOG, INFO, (char *)"Connect Server %d", i);
				connectServerThread(i);
				server_status[i] = CONN_CONNECTING;
				server_et[i]     = time_now;
			}
			break;

		case CONN_CONNECTING:
			if (isValidServerConnection(i)) {
				loginServer(i, bind_info+i);
				server_status[i] = CONN_VALIDATING;
				server_et[i]     = time_now;
			} else 
			if ((time_now - server_et[i]) > 10) {
				disconnectServer(i);
			}
			break;


		case CONN_VALIDATING:
			if ((time_now - server_et[i]) > 5) {
				disconnectServer(i);
			}
			break;

		case CONN_ACTIVE:
			if (elink_interval && ((time_now - last_elink) > elink_interval)) {
				last_elink = time_now;

				smpp_e.header.seq_num.set_value(getNextSeqNum());
				memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
				smpp_e.pack_msg(uc_out_smpp);
				sendServerSMPPMessage(i, uc_out_smpp);
			}
			break;

		default:
			break;
		}
	}
}

/************** ForServer
int SMClient::processClientEvent(int id)
{
	return 1;
}
**********************/

int SMClient::processServerEvent(int id)
{
	// read message from server
	unsigned char		uc_in_smpp[2000];

	SMPP_M_BindResp		smpp_br;
	SMPP_M_EnquireLink	smpp_e;
	SMPP_M_EnquireLinkResp	smpp_er;
	SMPP_M_SubmitSMResp	smpp_sr;
	SMPP_M_CancelSMResp	smpp_cr;
	SMPP_Header		smpp_hin;
	SMPP_Header		smpp_hou;

	int len = readServerSMPPMessage(id, uc_in_smpp, sizeof(uc_in_smpp)-1);
	if (len <= 0) {
		return -1;
	}

	if (debug_level > 100) {
		int	i;
		printf("RECV %02d : ", id);
		for (i=0; i<len; i++) {
			printf("%02X ", uc_in_smpp[i]);
		}
		printf("\n");

		logBuf((char *)"FROM SMSC", MSG_LOG, DEBUG, (unsigned char*)uc_in_smpp, len);
	}

	// process server message

	unsigned char		uc_out_smpp[2000];
	unsigned char		parameter[500];

	unsigned char		param[500];
	unsigned int		smpp_status;
	unsigned int		smpp_command;
	unsigned int		smpp_seqnum;
	int			ussd_type = 0;
	int			more_data = 0;
	int			i = 0;

	try {
		smpp_command = SMPP_GetCommandID(uc_in_smpp);
		smpp_status  = SMPP_GetCommandStatus(uc_in_smpp);
		smpp_seqnum  = SMPP_GetSeqNo(uc_in_smpp);

		switch (SMPP_GetCommandID(uc_in_smpp)) {
		case SMPP_BIND_TRANSMITTER_RESP:
			smpp_br.unpack_msg(uc_in_smpp);
			sprintf(log_msg_str, "SERVER %d: BIND_TRANSMITTER status %d seq_num %d", id, smpp_status, smpp_seqnum); LOGM;
			if (smpp_status != ESME_ROK) {
				loginfo(PROCESS_LOG, INFO, log_msg_str);
				error_monitor2(alarm_data,ALM_TCP_BREAK,1,3600);
				disconnectServer(id);
			} else {
				server_status[id] = CONN_ACTIVE;
				server_et[id]     = time(NULL);
			}
			break;

		case SMPP_UNBIND_RESP:
			sprintf(log_msg_str, "ID=%d, Status=%08X, Unbind    Resp Received", id, smpp_status); LOGM;
			break;

		case SMPP_SUBMIT_SM_RESP:
			sprintf(log_msg_str, "ID=%d, Status=%08X, SubmitSM  Resp Received", id, smpp_status); LOGM;
			smpp_sr.unpack_msg(uc_in_smpp);
			if (smpp_status != ESME_ROK) {
				sprintf(log_msg_str, "SERVER %d: SUBMIT_SM status %d", id, smpp_status); LOGM;
			}
			for (i=0; i<MAX_CLIENT_MAPPING_DATA; i++) {
				if (client_map[i].event_time && client_map[i].server_seqnum == smpp_seqnum) {
					if (client_map[i].client_id < MAX_CLIENT_NUM && client_map[i].client_id >= 0) {
						smpp_sr.header.seq_num.set_value(client_map[i].client_seqnum);
						memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
						smpp_sr.pack_msg(uc_out_smpp);
		// ForServer			sendClientSMPPMessage(id, uc_out_smpp);
					} else {
						smpp_sr.message_id.get_string((char *)client_map[i].message_id);
						send_reply_intm(client_map+i, smpp_status);
					}
					return 0;
				}
			}
			break;

		case SMPP_CANCEL_SM_RESP:
			sprintf(log_msg_str, "ID=%d, Status=%08X, CancelSM Resp Received", id, smpp_status); LOGM;
			smpp_cr.unpack_msg(uc_in_smpp);
			if (smpp_status != ESME_ROK) {
				sprintf(log_msg_str, "SERVER %d: SUBMIT_SM status %d", id, smpp_status); LOGM;
			}
			for (i=0; i<MAX_CLIENT_MAPPING_DATA; i++) {
				if (client_map[i].event_time && client_map[i].server_seqnum == smpp_seqnum) {
					if (client_map[i].client_id < MAX_CLIENT_NUM && client_map[i].client_id >= 0) {
						smpp_cr.header.seq_num.set_value(client_map[i].client_seqnum);
						memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
						smpp_cr.pack_msg(uc_out_smpp);
		// ForServer			sendClientSMPPMessage(id, uc_out_smpp);
					} else {
						send_reply_intm(client_map+i, smpp_status);
					}
					return 0;
				}
			}
			break;

		case SMPP_ENQUIRE_LINK:
		case SMPP_UNBIND:
			smpp_hin.unpack_header(uc_in_smpp);
			smpp_hou.seq_num.set_value(smpp_hin.seq_num.get_value());
			smpp_hou.command_id.set_value(smpp_hin.command_id.get_value()+0x80000000);
			memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
			smpp_hou.pack_header(uc_out_smpp);
			sendServerSMPPMessage(id, uc_out_smpp);
			break;

		case SMPP_ENQUIRE_LINK_RESP:
			printf("Enquire Link Response Received\n");
			break;

		case SMPP_GENERIC_NACK:
			printf("Generic Negative Acknowledgement Received\n");
			break;

		default:
			sprintf(log_msg_str, "ID=%d, Status=%08X, Default Message Received(MT=0x08X)", id, smpp_status, smpp_command);
			break;
		}
	} catch (SMPP_Exception smpp_err) {
		printf("SMPP Exception Catched!!!\n");
	}
	return 0;
}


unsigned long SMClient::getNextSeqNum()
{
	seq_num++;
	if (seq_num < 0x00000000 || seq_num > 0x7fffffff) {
		seq_num = 0;
	}
	return seq_num;
}


void SMClient::loginServer(int id, SMPPBind_S *data)
{
	SMPP_M_Bind		smpp;

	smpp.header.command_id.set_value(SMPP_BIND_TRANSMITTER);
	smpp.header.seq_num.set_value(getNextSeqNum());
	smpp.system_id.set_string((char *)data->system_id);
	smpp.password.set_string((char *)data->password);
	smpp.system_type.set_string((char *)data->system_type);
	smpp.interface_version.set_value(data->interface_version);
	smpp.addr_ton.set_value(data->addr_ton);
	smpp.addr_npi.set_value(data->addr_npi);
	smpp.address_range.set_string((char *)data->address_range);

	unsigned char		uc_smpp[2000];
	memset(uc_smpp, 0, sizeof(uc_smpp));
	smpp.pack_msg(uc_smpp);
	sendServerSMPPMessage(id, uc_smpp);

	return;
}


void SMClient::logoutServer(int id)
{
	SMPP_M_Unbind		smpp_ub;
	smpp_ub.header.seq_num.set_value(getNextSeqNum());

	unsigned char		uc_smpp[2000];
	memset(uc_smpp, 0, sizeof(uc_smpp));
	smpp_ub.pack_msg(uc_smpp);
	sendServerSMPPMessage(id, uc_smpp);

	return;
}


int SMClient::disconnectServer(int id)
{
	loginfo(PROCESS_LOG, INFO, (char *)"Server disconnected %d", id);
	error_monitor2(alarm_data,ALM_TCP_BREAK,1,3600);
	server_status[id] = CONN_IDLE;
	server_et[id]     = time(NULL);
        return TCPClient::disconnectServer(id);
}


int SMClient::processQueueMessage(intm_msg *msg, size_t msg_size)
{
	SMPP_M_SubmitSM		submit_sm;
	SMPP_M_CancelSM		cancel_sm;

	int		error = 0;
	int		param_len;
	unsigned int	sm_length = 0;
	int		smsc_id = -1;
	int		sm_type;
	int		msg_type;
	int		match;
	int		i;

	char		tmp_str[128];
	unsigned char	param[1024];
	unsigned char	sm_content[1024];
	unsigned char	flag[16];
	client_mapping_struct	cdata;

	//submit_sm.header.seq_num.set_value(getNextSeqNum());

	memset(&cdata, 0, sizeof(client_mapping_struct));	// 20030218
	cdata.client_id = -1;
	cdata.event_time = time(NULL);
	cdata.SPC     = msg->SPC;
	cdata.DPC     = msg->DPC;
	cdata.SUBSP   = msg->SUBSP;
	cdata.SUBDP   = msg->SUBDP;
	cdata.seq_num = msg->seq_num;
	cdata.server_seqnum = getNextSeqNum();
	//cdata.server_seqnum = submit_sm.header.seq_num.get_value();

	/****20030422***************************************
		Only support SUBMIT_SM and CANCEL_SM
		and SUBMIT_SM is set as default msg_type
	****************************************************/
	memset(param, 0, sizeof(param));
	if (msg->get_param(PROPERTY, param, &param_len)) {
		msg_type = param[0];
	} else {
		msg_type = MSG_TYPE_SUBMIT_SM;
	}

	memset(param, 0, sizeof(param));
	if (msg->get_param(SUB_TYPE, param, &param_len)) {
		sm_type = param[0];
	} else {
		if (msg->SPC) {
			sm_type = msg->SPC + 100;
		} else {
			sm_type = 0;
		}
	}


	if (msg_type == MSG_TYPE_SUBMIT_SM) {	// SUBMIT_SM
		char	msisdn[20];

		memset(msisdn, 0, sizeof(msisdn));
		if (msg->get_param(DEST_ADDR, param, &param_len)) {
			param[param_len] = 0x00;
			strncpy(msisdn, (char *)param, sizeof(msisdn) - 1);
		}

		submit_sm.header.seq_num.set_value(cdata.server_seqnum);

		if ((smsc_id = init_submit_sm(&submit_sm, sm_type, flag)) < 0) {
			printf("Error: Cannot initialize submit_sm content\n");
			send_reply_intm(&cdata, ESME_RUNKNOWNERR);
			return -1;
		} else {
			strcpy((char *)cdata.flag, (char *)flag);
		}

		if (msg->get_param(ID, param, &param_len)) {
			cdata.msg_seqnum[0] = param_len;
			memcpy(cdata.msg_seqnum+1, param, param_len);
		}

		sprintf(log_msg_str, "SMSC_ID = %d, FLAG = [%s]", smsc_id, flag); LOGM;

		if (server_status[smsc_id] != CONN_ACTIVE) {
			sprintf(log_msg_str, "Error: Destination server socket not in active state [%d]", server_status[smsc_id]); LOGM;
			send_reply_intm(&cdata, ESME_RBINDFAIL);
			return -1;
		}

		memset(param, 0, sizeof(param));
		if (!error && msg->get_param(DEST_ADDR, param, &param_len)) {
			param[param_len] = 0x00;
			if (cdata.flag[FLAG_POS_ADD_HOME_CODE] == '1') {
				sprintf(tmp_str, "%s%s", readCfg->get_s(GS_HOME_CODE), param);
				submit_sm.dest_addr.set_string(tmp_str);
			} else {
				submit_sm.dest_addr.set_string((char *)param);
			}
		} else {
			sprintf(log_msg_str, MODULE_NAME": Missing destination address"); LOGM;
			error++;
		}

		if (!error && msg->get_param(ESM_CLASS, param, &param_len)) {
			submit_sm.esm_class.set_value(param[0]);
		} else  {
			submit_sm.esm_class.set_value(0);
		}

		if (!error && msg->get_param(PRIORITY_FLAG, param, &param_len)) {
			submit_sm.priority_flag.set_value(param[0]);
		} else {
			submit_sm.priority_flag.set_value(0);
		}

		if (!error && msg->get_param(VALIDITY_PERIOD, param, &param_len)) {
			submit_sm.validity_period.set_string((char *)param);
		}

		if (!error && msg->get_param(DATA_CODING, param, &param_len)) {
			submit_sm.data_coding.set_value(param[0]);
		}

		if (!error && msg->get_param(SM_DEFAULT_MSG_ID, param, &param_len)) {
			submit_sm.sm_default_msg_id.set_value(param[0]);
		} else {
			submit_sm.sm_default_msg_id.set_value(0);
		}

		if (!error && msg->get_param(REGISTERED_DELIVERY, param, &param_len)) {
			submit_sm.registered_delivery.set_value(param[0]);
		} else {
			submit_sm.registered_delivery.set_value(0);
		}

		if (!error && msg->get_param(PROTOCOL_ID, param, &param_len)) {
			submit_sm.protocol_id.set_value(param[0]);
		} else {
			submit_sm.protocol_id.set_value(0);
		}

		memset(param, 0, sizeof(param));
		if (!error && msg->get_param(SM_SOURCE_ADDR, param, &param_len)) {
			param[param_len] = 0x00;
			submit_sm.source_addr.set_string((char *)param);
		} else {
			/***************************************	 v1.12 20020715
			if (submit_sm.source_addr[0] == 0x00) {
				printf(MODULE_NAME": Missing source address\n");
				error++;
			}
			****************************************/
		}

		if (!error && msg->get_param(DEST_ADDR_TON, param, &param_len)) {
			submit_sm.dest_addr_ton.set_value(param[0]);
		}

		if (!error && msg->get_param(DEST_ADDR_NPI, param, &param_len)) {
			submit_sm.dest_addr_npi.set_value(param[0]);
		}

		if (!error && msg->get_param(SOURCE_ADDR_TON, param, &param_len)) {
			submit_sm.source_addr_ton.set_value(param[0]);
		}

		if (!error && msg->get_param(SOURCE_ADDR_NPI, param, &param_len)) {
			submit_sm.source_addr_npi.set_value(param[0]);
		}

		if (!error && msg->get_param(SHORT_MESSAGE, param, &param_len)) {
			memset(sm_content, 0, sizeof(sm_content));
			if (cdata.flag[FLAG_POS_CONVERT_TO_UNI] == '1' && submit_sm.data_coding.get_value() == 8) {
				uni_util->big5_to_uni(param, param_len, sm_content, sm_length);
			} else 
			if (cdata.flag[FLAG_POS_CONVERT_TO_UNI] == '2' && submit_sm.data_coding.get_value() == 8) {
				uni_util->gb_to_uni(param, param_len, sm_content, sm_length);
			} else {
				memcpy(sm_content, param, param_len);
				sm_length = param_len;
			}
			submit_sm.short_message.set_content(sm_content, sm_length);
			submit_sm.sm_length.set_value(sm_length);

		} else {
			/*** Support short_message = null *******************
			     e.g. Virtual Message, pid=0, dcs=0, sm_length=0
			****************************************************/
			sm_length = 0;
		/*
			sprintf(log_msg_str, MODULE_NAME": Missing short message content"); LOGM;
			error++;
		*/
		}

		if (!error) {
			memset(tmp_str, 0, sizeof(tmp_str));
			submit_sm.source_addr.get_string(tmp_str);
			sprintf(log_msg_str, "SOURCE : [%s] %d %d", tmp_str, submit_sm.source_addr_ton.get_value(), submit_sm.source_addr_npi.get_value()); LOGM;

			memset(tmp_str, 0, sizeof(tmp_str));
			submit_sm.dest_addr.get_string(tmp_str);
			sprintf(log_msg_str, "DEST   : [%s] %d %d", tmp_str, submit_sm.dest_addr_ton.get_value(), submit_sm.dest_addr_npi.get_value()); LOGM;

			// v1.20: Display short message 
			sprintf(log_msg_str, "SHORT_MESSAGE : [%s] length [%d]", (char*)sm_content, sm_length); LOGM;

			unsigned char		uc_smpp[2000];
			memset(uc_smpp, 0, sizeof(uc_smpp));
			submit_sm.pack_msg(uc_smpp);

			for (i=0, match=-1; i<MAX_CLIENT_MAPPING_DATA; i++) {
				if (client_map[i].event_time == 0) {
					memcpy(client_map+i, &cdata, sizeof(client_mapping_struct));
					match = i;
					break;
				}
			}

			if (match < 0) {
				send_reply_intm(&cdata, ESME_RUNKNOWNERR);
				return -1;
			}

			sendServerSMPPMessage(smsc_id, uc_smpp);
		} else {
			send_reply_intm(&cdata, ESME_RUNKNOWNERR);
			sprintf(log_msg_str, MODULE_NAME": Proces Queue Msg Error - Not enough param for sending sm"); LOGM;
		}

	} else if (msg_type == MSG_TYPE_CANCEL_SM){	// CANCEL_SM
		cancel_sm.header.seq_num.set_value(cdata.server_seqnum);

		if ((smsc_id = init_cancel_sm(&cancel_sm, sm_type, flag)) < 0) {
			printf("Error: Cannot initialize cancel_sm content\n");
			send_reply_intm(&cdata, ESME_RUNKNOWNERR);
			return -1;
		} else {
			strcpy((char *)cdata.flag, (char *)flag);
		}

		if (msg->get_param(ID, param, &param_len)) {
			cdata.msg_seqnum[0] = param_len;
			memcpy(cdata.msg_seqnum+1, param, param_len);
		}

		sprintf(log_msg_str, "SMSC_ID = %d, FLAG = [%s]", smsc_id, flag); LOGM;

		if (server_status[smsc_id] != CONN_ACTIVE) {
			sprintf(log_msg_str, "Error: Destination server socket not in active state [%d]", server_status[smsc_id]); LOGM;
			send_reply_intm(&cdata, ESME_RBINDFAIL);
			return -1;
		}

		memset(param, 0, sizeof(param));
		if (!error && msg->get_param(MESSAGE_ID, param, &param_len)) {
			param[param_len] = 0x00;
			cancel_sm.message_id.set_string((char *)param);
		}

		memset(param, 0, sizeof(param));
		if (!error && msg->get_param(DEST_ADDR, param, &param_len)) {
			param[param_len] = 0x00;
			if (cdata.flag[FLAG_POS_ADD_HOME_CODE] == '1') {
				sprintf(tmp_str, "%s%s", readCfg->get_s(GS_HOME_CODE), param);
				cancel_sm.dest_addr.set_string(tmp_str);
			} else {
				cancel_sm.dest_addr.set_string((char *)param);
			}
		} else {
			sprintf(log_msg_str, MODULE_NAME": Missing destination address"); LOGM;
			error++;
		}


		memset(param, 0, sizeof(param));
		if (!error && msg->get_param(SM_SOURCE_ADDR, param, &param_len)) {
			param[param_len] = 0x00;
			cancel_sm.source_addr.set_string((char *)param);
		} else {
			/***************************************	 v1.12 20020715
			if (cancel_sm.source_addr[0] == 0x00) {
				printf(MODULE_NAME": Missing source address\n");
				error++;
			}
			****************************************/
		}

		if (!error && msg->get_param(DEST_ADDR_TON, param, &param_len)) {
			cancel_sm.dest_addr_ton.set_value(param[0]);
		}

		if (!error && msg->get_param(DEST_ADDR_NPI, param, &param_len)) {
			cancel_sm.dest_addr_npi.set_value(param[0]);
		}

		if (!error && msg->get_param(SOURCE_ADDR_TON, param, &param_len)) {
			cancel_sm.source_addr_ton.set_value(param[0]);
		}

		if (!error && msg->get_param(SOURCE_ADDR_NPI, param, &param_len)) {
			cancel_sm.source_addr_npi.set_value(param[0]);
		}

		if (!error) {
			memset(tmp_str, 0, sizeof(tmp_str));
			cancel_sm.source_addr.get_string(tmp_str);
			sprintf(log_msg_str, "SOURCE : [%s] %d %d", tmp_str, cancel_sm.source_addr_ton.get_value(), cancel_sm.source_addr_npi.get_value()); LOGM;

			memset(tmp_str, 0, sizeof(tmp_str));
			cancel_sm.dest_addr.get_string(tmp_str);
			sprintf(log_msg_str, "DEST   : [%s] %d %d", tmp_str, cancel_sm.dest_addr_ton.get_value(), cancel_sm.dest_addr_npi.get_value()); LOGM;

			unsigned char		uc_smpp[2000];
			memset(uc_smpp, 0, sizeof(uc_smpp));
			cancel_sm.pack_msg(uc_smpp);

			for (i=0, match=-1; i<MAX_CLIENT_MAPPING_DATA; i++) {
				if (client_map[i].event_time == 0) {
					memcpy(client_map+i, &cdata, sizeof(client_mapping_struct));
					match = i;
					break;
				}
			}

			if (match < 0) {
				send_reply_intm(&cdata, ESME_RUNKNOWNERR);
				return -1;
			}

			sendServerSMPPMessage(smsc_id, uc_smpp);
		} else {
			send_reply_intm(&cdata, ESME_RUNKNOWNERR);
			sprintf(log_msg_str, MODULE_NAME": Proces Queue Msg Error - Not enough param for sending sm"); LOGM;
		}
	}

	return 1;
}


int SMClient::init_submit_sm(SMPP_M_SubmitSM *submit_sm, unsigned int my_mtype, unchar *flag)
{
	int	smsc_id = -1;
	int	found = -1;
	int	i;
	char	tmp_str[128];

//	printf("MTYPE = %02X\n", my_mtype);
	for (i=0; i<MAX_PROFILE_NUM; i++) {
		if (profile[i].mtype == my_mtype) {
			found = i;
			break;
		} else 
		if (profile[i].mtype == 0 && profile[i].flag[0] && found == -1) {
			found = i;
		}
	}

	if (found != -1) {
		submit_sm->dest_addr_npi.set_value(profile[found].dest_addr_npi);
		submit_sm->dest_addr_ton.set_value(profile[found].dest_addr_ton);
		submit_sm->source_addr_npi.set_value(profile[found].source_addr_npi);
		submit_sm->source_addr_ton.set_value(profile[found].source_addr_ton);
		submit_sm->source_addr.set_string((char *)profile[found].source_addr);
		submit_sm->validity_period.set_string((char *)profile[found].validity_period);
		submit_sm->data_coding.set_value(profile[found].data_coding);
		smsc_id = profile[found].server_index;
		strcpy((char *)flag, (char *)profile[found].flag);
	}

	memset(tmp_str, 0, sizeof(tmp_str));
	submit_sm->source_addr.get_string(tmp_str);

	sprintf(log_msg_str, "INIT_SUBMIT_SM: smsc_id = %d [%02X %02X %02X %02X [%s]]", smsc_id, 
		submit_sm->source_addr_ton.get_value(), 
		submit_sm->source_addr_npi.get_value(), 
		submit_sm->dest_addr_ton.get_value(), 
		submit_sm->dest_addr_npi.get_value(), tmp_str); LOGM;

	return smsc_id;
}


int SMClient::init_cancel_sm(SMPP_M_CancelSM *cancel_sm, unsigned int my_mtype, unchar *flag)
{
	int	smsc_id = -1;
	int	found = -1;
	int	i;
	char	tmp_str[128];

//	printf("MTYPE = %02X\n", my_mtype);
	for (i=0; i<MAX_PROFILE_NUM; i++) {
		if (profile[i].mtype == my_mtype) {
			found = i;
			break;
		} else 
		if (profile[i].mtype == 0 && profile[i].flag[0] && found == -1) {
			found = i;
		}
	}

	if (found != -1) {
		cancel_sm->dest_addr_npi.set_value(profile[found].dest_addr_npi);
		cancel_sm->dest_addr_ton.set_value(profile[found].dest_addr_ton);
		cancel_sm->source_addr_npi.set_value(profile[found].source_addr_npi);
		cancel_sm->source_addr_ton.set_value(profile[found].source_addr_ton);
		cancel_sm->source_addr.set_string((char *)profile[found].source_addr);
		smsc_id = profile[found].server_index;
		strcpy((char *)flag, (char *)profile[found].flag);
	}

	memset(tmp_str, 0, sizeof(tmp_str));
	cancel_sm->source_addr.get_string(tmp_str);

	sprintf(log_msg_str, "INIT_CANCEL_SM: smsc_id = %d [%02X %02X %02X %02X [%s]]", smsc_id, 
		cancel_sm->source_addr_ton.get_value(), 
		cancel_sm->source_addr_npi.get_value(), 
		cancel_sm->dest_addr_ton.get_value(), 
		cancel_sm->dest_addr_npi.get_value(), tmp_str); LOGM;

	return smsc_id;
}


int SMClient::send_reply_intm(client_mapping_struct *data, int cause_code)
{
	intm_msg	send_msg;
	unchar		param[16];
	int		send_flag = 0;
	int		my_qid, my_mtype;

	sprintf(log_msg_str, "FLAG = [%s], cause_code = %d", data->flag, cause_code); LOGM;
	if (data->flag[FLAG_POS_SEND_REPLY] == '1') {
		send_msg.init_msg(data->DPC, data->SPC, data->SUBDP, data->SUBSP, data->seq_num, ACK);
		param[0] = cause_code;
		send_msg.add_param(STATUS, param, 1);
		send_flag = 1;
	} else 
	if (data->flag[FLAG_POS_SEND_REPLY] == '2') {
		if (cause_code != 0) {
			send_msg.init_msg(data->DPC, data->SPC, data->SUBDP, data->SUBSP, data->seq_num, ACK);
			param[0] = cause_code;
			send_msg.add_param(CAUSE_CODE, param, 1);
			send_flag = 1;
		} else {
			send_msg.init_msg(data->DPC, data->SPC, data->SUBDP, data->SUBSP, data->seq_num, ACK);
			send_flag = 1;
		}
	}

	if (send_flag == 1) {	/* v1.13 20020725 */
		if (data->msg_seqnum[0]) {
			send_msg.add_param(ID, data->msg_seqnum+1, data->msg_seqnum[0]);
		}
		if (data->message_id[0]) {
			send_msg.add_param(MESSAGE_ID, data->message_id, strlen((char *)data->message_id));
		}
		send_vapp(&send_msg);
	}
	memset(data, 0, sizeof(client_mapping_struct));

	return 1;
}


int SMClient::send_vapp(intm_msg *intm)
{
	int module_id = intm->DPC;
	if (module_id < 0 || module_id >= MAX_APP_QUEUE || !appq[module_id]) {
		module_id = 0;
	}
	if (!appq[module_id]) {
		loginfo(PROCESS_LOG, MAJOR, (char *)"unable to route message to correct queue");
		error_monitor2(alarm_data,ALM_MSGQ_ERR,1,3600);
		return -1;
	}

	appq[module_id]->send(intm, intm->len_indicator+2, mtype[module_id]);

	return 0;
}


int SMClient::sendServerSMPPMessage(int id, unsigned char *buffer)
{
	int	i, len;

	if (debug_level > 100) {
		memcpy(&len, buffer, sizeof(int));
		printf("SEND %02d : ", id);
		for(i=0; i<len; i++) {
			printf("%02X ", buffer[i]);
		}
		printf("\n");

		logBuf((char*)"TO SMSC", MSG_LOG, DEBUG,(unsigned char*) buffer, len);
	}

	return SMPP_TCPClient::sendServerSMPPMessage(id, buffer);
}
