#ifndef SM_CLIENT_H
#define SM_CLIENT_H

#include <poll.h>
#include "IPC_Memory.h"
#include "IPC_Queue.h"
#include "SysParam.h"
#include "LogFile.h"
#include "SMPP_incl.h"
#include "SMPP_TCPServer.h"
#include "SMPP_TCPClient.h"
#include "INTM_Message.h"
#include "ReadCfg.h"
#include "ReadCfgTag.h"
#include "SMClientCfg.h"
#include "UniCode_Util.h"

/***** Please adjust these values *****/
#define MAX_SERVER_NUM          4
#define MAX_CLIENT_NUM          4
/*************************************/
#define MAX_PROFILE_NUM		MAX_SERVER_NUM*4
#define MAX_APP_QUEUE		60

#define LOGM	if (debug_level) logFile->log_msg(log_msg_str, debug_level);

#define FLAG_POS_ADD_HOME_CODE		0
#define FLAG_POS_SEND_REPLY		1
#define FLAG_POS_CONVERT_TO_UNI		2

#define MAX_CLIENT_MAPPING_DATA         4096

#define MSG_TYPE_SUBMIT_SM		0
#define	MSG_TYPE_CANCEL_SM		1

typedef struct {
        unsigned int    server_seqnum;
        unsigned int    client_seqnum;
        int             client_id;
        time_t          event_time;
        unsigned char   SPC;
        unsigned char   DPC;
        unsigned short  SUBSP;
        unsigned short  SUBDP;
        unsigned short  seq_num;
        unsigned char   flag[9];
	unsigned char	msg_seqnum[12];
	unsigned char	message_id[65];
} client_mapping_struct;


class SMClient : public SMPP_TCPClient {
private:
	enum {
		CONN_ERROR = -1,
		CONN_IDLE  = 0,
		CONN_CONNECTING,
		CONN_VALIDATING,
		CONN_ACTIVE
	};

	IPC_Queue	*smclntq;
	IPC_Queue	*appq[MAX_APP_QUEUE];
	int		queue[MAX_APP_QUEUE];
	int		mtype[MAX_APP_QUEUE];
	int		listen_port;
	int		server_status[MAX_SERVER_NUM];
	time_t		server_et[MAX_SERVER_NUM];
	ReadCfg		*readCfg;
	SMClientCfg	*smclntCfg;
	LogFile		*logFile;
	char		config_file[128];
	char		log_msg_str[1024];
	SMPPBind_S	bind_info[MAX_SERVER_NUM];
	SMPPProfile_S	profile[MAX_PROFILE_NUM];
	client_mapping_struct	client_map[MAX_CLIENT_MAPPING_DATA];
	UniCode_Util	*uni_util;
	IPC_Memory                      *SysMem;

public:
	int		quit;
	unsigned long	seq_num;
	int		debug_level;
	int		reload_flag;

public:
	SMClient();
	~SMClient();

	unsigned long getNextSeqNum();

	void set_signal();
	void loginServer(int id, SMPPBind_S *data);
	void logoutServer(int id);
	void waitEvent();
	void init(int argc, char **argv);

	void check_timeout(time_t time_now);
	int  readAppRouteConfig(char *filename);
//	int  processClientEvent(int id);
	int  processServerEvent(int id);
	int  processQueueMessage(intm_msg *msg, size_t msg_size);
	int  disconnectServer(int id);
	int  init_submit_sm(SMPP_M_SubmitSM *submit_sm, unsigned int mtype, unchar *flag);
	int  init_cancel_sm(SMPP_M_CancelSM *cancel_sm, unsigned int mtype, unchar *flag);
	int  send_reply_intm(client_mapping_struct *data, int cause_code);
	int  send_vapp(intm_msg *intm);
	int  sendServerSMPPMessage(int id, unsigned char *buffer);

};

#endif SM_CLIENT_H
