#define MODULE_NAME	"SMCLIENT_CFG"
#define VERSION_NUM	"0.10"
#define REVISION_DATE	"20011031"
static char version[]="\n" MODULE_NAME " Version " VERSION_NUM " Copyright (c)  "REVISION_DATE "\n";
#define	DEBUG_LINE	printf(__FILE__": DEBUG_LINE %04d\n", __LINE__);

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SysUtil.h"
#include "SMClientCfg.h"


SMClientCfg::SMClientCfg()
{
}


SMClientCfg::~SMClientCfg()
{
}

int SMClientCfg::ReadConfig(const char *filename, SMPPBind_S *info, SMPPProfile_S *profile, int max_size)
{
	FILE	*fp;
	char	buffer[4096];
	int	count;
	char	token[4096];
	int	match_token = 0;
	int	line = 0;

	char	system_id[128];
	char	password[128];
	char	system_type[128];
	char	address_range[128];
	char	source_addr[128];
	int	interface_version;
	int	addr_ton;
	int	addr_npi;
	int	index;
	int	dest_addr_ton;
	int	dest_addr_npi;
	int	mtype;
	unsigned char	validity_period[128];
	unsigned char	param1[128];
	int	data_coding;
	char	flag[16];

	if (!getcfgpath(buffer, filename)) {
		return -1;
	}

	if ((fp = fopen(buffer, "r")) == NULL) {
		printf(MODULE_NAME":Can't open %s\n",buffer);
		perror(MODULE_NAME":fopen() ");
		return -1;
	}

	printf(MODULE_NAME":Reading %s...\n", buffer);

	strcpy(token, "[SMSC_INFO]");
	match_token = 0;
	line = 0;

	while(memset(buffer, 0, sizeof(buffer)) && fgets(buffer, sizeof(buffer), fp)) {
		if (buffer[0] == ';' || buffer[0] == '#') {
			continue;
		}

		if (match_token == 0 && !strstr(buffer, token)) {
			continue;
		} else 
		if (match_token == 0 && line == 0) {
			match_token = 1;
			continue;
		} else {
			/***  read config ****/
		}

		index = -1;
		memset(system_id, 0, sizeof(system_id));
		memset(password, 0, sizeof(password));
		memset(system_type, 0, sizeof(system_type));
		interface_version = 0;
		addr_ton = 0;
		addr_npi = 0;
		memset(address_range, 0, sizeof(address_range));
		if ((count = sscanf(buffer,"%d %s %s %s %d %d %d %s", &index, system_id, password, system_type, &interface_version, &addr_ton, &addr_npi, address_range)) >= 5) {
			if (index >= 0 && index < max_size) {
				line++;
				if (!strcmp(system_id, "-")) {
					memset(system_id, 0, sizeof(system_id));
				}
				if (!strcmp(password, "-")) {
					memset(password, 0, sizeof(password));
				}
				if (!strcmp(system_type, "-")) {
					memset(system_type, 0, sizeof(system_type));
				}
				strncpy((char *)info[index].system_id, system_id, 16);
				strncpy((char *)info[index].password, password, 9);
				strncpy((char *)info[index].system_type, system_type, 13);
				info[index].interface_version = interface_version;
				info[index].addr_ton = addr_ton;
				info[index].addr_npi = addr_npi;
				strncpy((char *)info[index].address_range, address_range, 13);
				printf("BIND INFO %d: [%s] [%s] [%s] %d %d %d [%s]\n", index,
					info[index].system_id, 
					info[index].password, 
					info[index].system_type, 
					info[index].interface_version, 
					info[index].addr_ton, 
					info[index].addr_npi, 
					info[index].address_range);
			} else {
				fprintf(stderr, MODULE_NAME":Incorrect index (%d) read\n", index);
			}
		} else {
			break;
		}
	}


	rewind(fp);

	strcpy(token, "[PROFILE_INFO]");
	match_token = 0;
	line = 0;

	while(memset(buffer, 0, sizeof(buffer)) && fgets(buffer, sizeof(buffer), fp)) {
		if (buffer[0] == ';' || buffer[0] == '#') {
			continue;
		}

		if (match_token == 0 && !strstr(buffer, token)) {
			continue;
		} else 
		if (match_token == 0 && line == 0) {
			match_token = 1;
			continue;
		} else {
			/***  read config ****/
		}

		index = -1;
		mtype = 0;
		addr_ton = 0;
		addr_npi = 0;
		dest_addr_ton = 0;
		dest_addr_npi = 0;
		data_coding = 0;
		memset(source_addr, 0, sizeof(source_addr));
		memset(validity_period, 0, sizeof(validity_period));
		memset(param1, 0, sizeof(param1));
		memset(flag, 0, sizeof(flag));
		if ((count = sscanf(buffer,"%d %d %s %d %d %d %d %s %s %d", &mtype, &index, flag, &addr_ton, &addr_npi, &dest_addr_ton, &dest_addr_npi, source_addr, param1, &data_coding)) >= 8) {

			if (index >= 0 && index < (max_size*4)) {
				if (!strcmp(source_addr, "-")) {	/* v1.12 20020715 */
					memset(source_addr, 0, sizeof(source_addr));
				}

				profile[line].mtype = mtype;
				profile[line].server_index = index;
				profile[line].source_addr_ton = addr_ton;
				profile[line].source_addr_npi = addr_npi;
				profile[line].dest_addr_ton = dest_addr_ton;
				profile[line].dest_addr_npi = dest_addr_npi;
				strncpy((char *)profile[line].flag, flag, 8);
				strncpy((char *)profile[line].source_addr, source_addr, 21);

				get_validity_period_str((char *)param1, (char *)validity_period);
				strcpy((char *)profile[line].validity_period, (char *)validity_period);
				profile[line].data_coding = data_coding;

				printf("PROFILE: %d %d %s %d %d %d %d [%s] [%s] %d\n", 
					profile[line].mtype,
					profile[line].server_index,
					profile[line].flag, 
					profile[line].source_addr_ton,
					profile[line].source_addr_npi,
					profile[line].dest_addr_ton,
					profile[line].dest_addr_npi,
					profile[line].source_addr,
					profile[line].validity_period,
					profile[line].data_coding);
				line++;
			} else {
				fprintf(stderr, MODULE_NAME":Incorrect index (%d) read\n", index);
			}
		} else {
			break;
		}
	}

	fclose(fp);

	return 0;
}

char *SMClientCfg::get_validity_period_str(char *str, char *validity_period_str)
{
	int i=0,rc=0;
	validity_period_str[0]=0;
	if (str && strlen(str)>0 && strlen(str)<=12) {
		for (i=0;i<strlen(str);i++) {
			if ( str[i] < '0' || str[i] > '9' ) {
				rc=-1;
				break;
			}
		}
	} else {
		rc=-2;
	}
	if (rc != 0 || atoi(str)==0) {
		validity_period_str[0]=0;
	} else {
		sprintf(validity_period_str, "%012s000R", str);
	}
	return validity_period_str;

}

