/********************************************************************

	Filename	: CDRLogger.h
	Description	: CDR Logger class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation

********************************************************************/

#include <stdlib.h>
#ifndef CDRLOGGER_H
#define CDRLOGGER_H

#include <stdio.h>
#include "Logger.h"
#include "IPC_Queue.h"

class CDRLogger : public Logger {
public:		// functions

	/**
	*	@memo Description:
	*		Default constructor
	*/
	CDRLogger();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~CDRLogger();

	/**
	*	@memo Description:
	*		Set application type of VAPP for CDR
	*	@exception
	*		None
	*	@return
	*		0
	*	@param app_type (input)
	*		Application type (1 - 15) for VAPP.
	*		CDR Logger will read log message queue with message type (4000 + app_type).
	*/
	int setAppType(int app_type);

	/**
	*	@memo Description:
	*		Write information in Log.  Invoke automatically if message received.
	*		This function must be implemented in the derived class.
	*	@exception
	*		None
	*	@return
	*		Nil
	*	@param f (input)
	*		File pointer of the opened log file
	*	@param msg (input)
	*		Pointer to the log data message which contains information to be logged into file
	*/
	virtual void write_LOG(FILE *f, unsigned char *msg) = 0;
};

#endif	/* CDRLOGGER_H */
