/********************************************************************

	Filename	: LogUtil.h
	Description	: LOG Utility Header

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011201	Michael Chen: Modified for dynamic module name

********************************************************************/

#ifndef LOGUTIL_H
#define LOGUTIL_H

#define MAX_LOG_DATA	2000

/***** LOG Type *****/
#define PROCESS_LOG	8191
#define SDS_LOG		8190
#define MSG_LOG		8189
#define TRACE_LOG	8188
#define REMOTE_LOG	8187

#define APP1_CDR	4001
#define APP2_CDR	4002
#define APP3_CDR	4003
#define APP4_CDR	4004
#define APP5_CDR	4005
#define APP6_CDR	4006
#define APP7_CDR	4007
#define APP8_CDR	4008
#define APP9_CDR	4009
#define APP10_CDR	4010
#define APP11_CDR	4011
#define APP12_CDR	4012
#define APP13_CDR	4013
#define APP14_CDR	4014
#define APP15_CDR	4015


/***** LOG Level *****/
#define OLD_LOGGER
#ifdef OLD_LOGGER
#define MAJOR		0
#define MINOR		1000
#define WARNING		2000
#define INFO		3000
#define TRACE		4000
#define DEBUG		5000
#else
#define DEBUG		0
#define TRACE		1000
#define INFO		2000
#define WARNING		3000
#define MINOR		4000
#define MAJOR		5000
#define CRITICAL	6000
#define FATAL		7000
#endif


/**
*	@memo Description:
*		Initiate LOG module ID in SYSTEM.LOG / MSG.LOG
*	@exception
*		None
*	@return
*		Nil
*	@param module_id (input)
*		Module ID
*/
void init_log(int module_id);

/**
*	@memo Description:
*		Initiate LOG module name in SYSTEM.LOG / MSG.LOG
*	@exception
*		None
*	@return
*		Nil
*	@param name (input)
*		Module name (maximum 8 characters are allowed)
*/
void init_log(char *name);

/**
*	@memo Description:
*		Log system event message
*	@exception
*		GeneralErrorException - cannot initialize log message queue
*	@return
*		0
*	@param type (input)
*		Log type, e.g. PROCESS_LOG for SYSTEM.LOG, MSG_LOG for MSG.LOG
*	@param level (input)
*		Log level, e.g. MAJOR, MINOR, WARNING, INFO, TRACE, DEBUG
*	@param format (input)
*		Format and arguments to be logged
*/
int loginfo(int type, int level, char *format, ...);

/**
*	@memo Description:
*		Log hexadecimal representation of message
*	@exception
*		GeneralErrorException - cannot initialize log message queue
*	@return
*		0
*	@param header (input)
*		Log header, e.g. "TO QM", "FROM QM"
*	@param type (input)
*		Log type, e.g. MSG_LOG for MSG.LOG, PROCESS_LOG for SYSTEM.LOG
*	@param level (input)
*		Log level, e.g. MAJOR, MINOR, WARNING, INFO, TRACE, DEBUG
*	@param data (input)
*		Message content to be logged
*	@param len (input)
*		Length of the message
*/
int logBuf(char *header, int type, int level, unsigned char *data, int len);

/**
*	@memo Description:
*		Log hexadecimal representation of message received from other module
*	@exception
*		GeneralErrorException - cannot initialize log message queue
*	@return
*		0
*	@param app (input)
*		Module ID where the message is received from
*	@param type (input)
*		Log type, e.g. MSG_LOG for MSG.LOG, PROCESS_LOG for SYSTEM.LOG
*	@param level (input)
*		Log level, e.g. MAJOR, MINOR, WARNING, INFO, TRACE, DEBUG
*	@param data (input)
*		Message content to be logged
*	@param len (input)
*		Length of the message
*/
int from_logBuf(int app, int type, int level, unsigned char *data, int len);

/**
*	@memo Description:
*		Log hexadecimal representation of message sent to other module
*	@exception
*		GeneralErrorException - cannot initialize log message queue
*	@return
*		0
*	@param app (input)
*		Module ID where the message is sent to
*	@param type (input)
*		Log type, e.g. MSG_LOG for MSG.LOG, PROCESS_LOG for SYSTEM.LOG
*	@param level (input)
*		Log level, e.g. MAJOR, MINOR, WARNING, INFO, TRACE, DEBUG
*	@param data (input)
*		Message content to be logged
*	@param len (input)
*		Length of the message
*/
int to_logBuf(int app, int type, int level, unsigned char *data, int len);

#endif		/* LOGUTIL_H */
