/********************************************************************

	Filename	: SMPP_TCPClient.h
	Description	: SMPP TCP Client Class header file

	History:
	------------------------------------------------------------
	20010710	Michael Chen: First Creation

********************************************************************/

#ifndef SMPP_TCPCLIENT_H
#define SMPP_TCPCLIENT_H

#include "TCPClient.h"

class SMPP_TCPClient : public TCPClient {
public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	SMPP_TCPClient();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~SMPP_TCPClient();

	/**
	*	@memo Description:
	*		Read SMPP message from a server.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in reading,
	*		otherwise return number of bytes read
	*	@param id (input)
	*		Server ID
	*	@param buffer (output)
	*		SMPP message to be read
	*	@param max_len (input)
	*		Maximum size of the buffer to be read
	*/
	virtual int readServerSMPPMessage(int id, unsigned char *buffer, size_t max_len);

	/**
	*	@memo Description:
	*		Send SMPP message to a server.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in sending,
	*		otherwise return number of bytes sent
	*	@param id (input)
	*		Server ID
	*	@param msg (input)
	*		SMPP message to be sent
	*/
	virtual int sendServerSMPPMessage(int id, unsigned char *buffer);

	/**
	*	@memo Description:
	*		Process server event, e.g. incoming message or disconnection from server.
	*		It is called automatically when there is a server event when pollServerEvent() is called.
	*		It must be overrided and user should call readServerSMPPMessage() to read message
	*		and return -1 when readServerSMPPMessage() returns <= 0.
	*	@exception
	*		None
	*	@return
	*		-1 when there is error in event handling, e.g. disconnection case,
	*		0 when successful
	*	@param id (input)
	*		Server ID
	*/
	virtual int processServerEvent(int id) = 0;
};

#endif /* SMPP_TCPCLIENT_H */
