/********************************************************************

	Filename	: SMPP_TCPServer.h
	Description	: SMPP TCP Server Class header file

	History:
	------------------------------------------------------------
	20010710	Michael Chen: First Creation

********************************************************************/

#ifndef SMPP_TCPSERVER_H
#define SMPP_TCPSERVER_H

#include "TCPServer.h"

class SMPP_TCPServer : public TCPServer {
public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	SMPP_TCPServer();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	virtual ~SMPP_TCPServer();

	/**
	*	@memo Description:
	*		Read SMPP message from a client.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in reading,
	*		otherwise return number of bytes read
	*	@param id (input)
	*		Client ID
	*	@param buffer (output)
	*		SMPP message to be read
	*	@param max_len (input)
	*		Maximum size of the buffer to be read
	*/
	virtual int readClientSMPPMessage(int id, unsigned char *buffer, size_t max_len);

	/**
	*	@memo Description:
	*		Send SMPP message to a client.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in sending,
	*		otherwise return number of bytes sent
	*	@param id (input)
	*		Client ID
	*	@param msg (input)
	*		SMPP message to be sent
	*/
	virtual int sendClientSMPPMessage(int id, unsigned char *buffer);

	/**
	*	@memo Description:
	*		Process client event, e.g. incoming message or disconnection from client.
	*		It is called automatically when there is a server event when pollClientEvent() is called.
	*		It must be overrided and user should call readClientSMPPMessage() to read message
	*		and return -1 when readClientSMPPMessage() returns <= 0.
	*	@exception
	*		None
	*	@return
	*		-1 when there is error in event handling, e.g. disconnection case,
	*		0 when successful
	*	@param id (input)
	*		Client ID
	*/
	virtual int processClientEvent(int id) = 0;
};

#endif /* SMPP_TCPSERVER_H */
