/********************************************************************

	Filename	: GeneralErrorException.h
	Description	: General Error Exception Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation
	20011206	Michael Chen: Added operator<<

********************************************************************/

#ifndef GENERALERROREXCEPTION_H
#define GENERALERROREXCEPTION_H

#include <iostream.h>

class GeneralErrorException {
protected:	// variables
	/**
	*	@memo Description:
	*		Error code
	*/
	int	error_code;

	/**
	*	@memo Description:
	*		Error message
	*/
	char	error_msg[1024];

public:		// functions
	/**
	*	@memo Description:
	*		Default constructor
	*/
	GeneralErrorException();

	/**
	*	@memo Description:
	*		Constructor with error message setting
	*	@param format (input)
	*		Format and arguments to be printed to the error message
	*/
	GeneralErrorException(char *format, ...);

	/**
	*	@memo Description:
	*		Constructor with error code setting
	*	@param code (input)
	*		Error code to be set
	*/
	GeneralErrorException(int code);

	/**
	*	@memo Description:
	*		Get error message
	*	@exception
	*		None
	*	@return
	*		Pointer to character arrays of the error message
	*/
	char* getMessage();

	/**
	*	@memo Description:
	*		Get error code
	*	@exception
	*		None
	*	@return
	*		Error code
	*/
	int getErrorCode();

	/**
	*	@memo Description:
	*		Print error message to the output stream,
	*		e.g., cout << err where err is a GeneralErrorException object.
	*	@exception
	*		None
	*	@return
	*		Reference to the output stream
	*/
	friend ostream& operator<<(ostream& output, GeneralErrorException outputGEE);
};

#endif /* GENERALERROREXCEPTION_H */
