/********************************************************************

	Filename	: INTM_TCPClient.h
	Description	: INTM TCP Client Class header file

	History:
	------------------------------------------------------------
	20010728	Michael Chen: First Creation

********************************************************************/

#ifndef INTM_TCPCLIENT_H
#define INTM_TCPCLIENT_H

#include "TCPClient.h"
#include "INTM_Message.h"

class INTM_TCPClient : public TCPClient {
public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	INTM_TCPClient();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~INTM_TCPClient();

	/**
	*	@memo Description:
	*		Read INTM message from a server.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in reading,
	*		otherwise return number of bytes read
	*	@param id (input)
	*		Server ID
	*	@param msg (output)
	*		INTM message to be read
	*/
	virtual int readServerINTMMessage(int id, intm_msg *msg);

	/**
	*	@memo Description:
	*		Send INTM message to a server.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in sending,
	*		otherwise return number of bytes sent
	*	@param id (input)
	*		Server ID
	*	@param msg (input)
	*		INTM message to be sent
	*/
	virtual int sendServerINTMMessage(int id, intm_msg *msg);

	/**
	*	@memo Description:
	*		Process server event, e.g. incoming message or disconnection from server.
	*		It is called automatically when there is a server event when pollServerEvent() is called.
	*		It must be overrided and user should call readServerINTMMessage() to read message
	*		and return -1 when readServerINTMMessage() returns <= 0.
	*	@exception
	*		None
	*	@return
	*		-1 when there is error in event handling, e.g. disconnection case,
	*		0 when successful
	*	@param id (input)
	*		Server ID
	*/
	virtual int processServerEvent(int id) = 0;
};

#endif /* INTM_TCPCLIENT_H */
