/********************************************************************

	Filename	: INTM_TCPServer.h
	Description	: INTM TCP Server Class header file

	History:
	------------------------------------------------------------
	20010728	Michael Chen: First Creation

********************************************************************/

#ifndef INTM_TCPSERVER_H
#define INTM_TCPSERVER_H

#include "TCPServer.h"
#include "INTM_Message.h"

class INTM_TCPServer : public TCPServer {
public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	INTM_TCPServer();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~INTM_TCPServer();

	/**
	*	@memo Description:
	*		Read INTM message from a client.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in reading,
	*		otherwise return number of bytes read
	*	@param id (input)
	*		Client ID
	*	@param msg (output)
	*		INTM message to be read
	*/
	virtual int readClientINTMMessage(int id, intm_msg *msg);

	/**
	*	@memo Description:
	*		Send INTM message to a client.
	*		It can be overrided.
	*	@exception
	*		None
	*	@return
	*		0 when there is error in sending,
	*		otherwise return number of bytes sent
	*	@param id (input)
	*		Client ID
	*	@param msg (input)
	*		INTM message to be sent
	*/
	virtual int sendClientINTMMessage(int id, intm_msg *msg);

	/**
	*	@memo Description:
	*		Process client event, e.g. incoming message or disconnection from client.
	*		It is called automatically when there is a client event when pollClientEvent() is called.
	*		It must be overrided and user should call readClientINTMMessage() to read message
	*		and return -1 when readClientINTMMessage() returns <= 0.
	*	@exception
	*		None
	*	@return
	*		-1 when there is error in event handling, e.g. disconnection case,
	*		0 when successful
	*	@param id (input)
	*		Client ID
	*/
	virtual int processClientEvent(int id) = 0;
};

#endif /* INTM_TCPSERVER_H */
