/********************************************************************

	Filename	: IPC_Semaphore.h
	Description	: IPC Semaphore Class header file

	History:
	------------------------------------------------------------
	20010701	Michael Chen: First Creation

********************************************************************/

#include <stdio.h>
#ifndef IPC_SEMAPHORE_H
#define IPC_SEMAPHORE_H

#include "IPC_Object.h"

class IPC_Semaphore : public IPC_Object {
public:
	/**
	*	@memo Description:
	*		Default constructor
	*/
	IPC_Semaphore();

	/**
	*	@memo Description:
	*		Default destructor
	*/
	~IPC_Semaphore();

	/**
	*	@memo Description:
	*		Initialize and attach an inter-process communication (IPC) semaphore
	*	@exception
	*		GeneralErrorException - initialization failure
	*	@return
	*		Nil
	*	@param key (input)
	*		IPC unique key for semaphore, valid values from 0x00 to 0x7F
	*	@param flag (input)
	*		If (flag & IPC_CREAT) is true,
	*		semaphore will be created with permission given in other bits of the flag
	*/
	void init(int key, int flag);

	/**
	*	@memo Description:
	*		Detach and destroy a IPC semaphore
	*	@exception
	*		GeneralErrorException - destroy failure
	*	@return
	*		Nil
	*/
	void destroy();

	/**
	*	@memo Description:
	*		Lock the semaphore.
	*		The function suspends when the semaphore is already locked by another process.
	*		The function returns immediately once the semaphore is in unlock state.
	*	@exception
	*		None
	*	@return
	*		Nil
	*/
	void lock();

	/**
	*	@memo Description:
	*		Unlock the semaphore.
	*	@exception
	*		None
	*	@return
	*		Nil
	*/
	void unlock();
};

#endif		/* IPC_SEMAPHORE_H */
