/********************************************************************

	Filename	: SMPP.h
	Description	: SMPP General include header file
			  - Constant definition specified in SMPP 3.4

	History:
	------------------------------------------------------------
	20010707	fw: First Creation
	20010808	kei: SMPP_GetSeqNo and SMPP_GetCommandStatus added, 
			Final Message State for Delivery Receipt Format (Status Report) added
	20011019	kei: Add pack_msg of DeliverSM for diff. DR
	20011020	fw: Exchange value of SMPP_DRFT_USERDEF and SMPP_DRFT_NORMAL
	20011123	kei: Add OP for PTC-SMX as GWNO of OA and DA
	20020109	kei: Add HLR optional param
	20020703	fw: Add Card Type

********************************************************************/

#ifndef SMPP_H
#define SMPP_H

#include <stdlib.h>


////////////////////////////////////////////////////////////////////////
//
//  SMPP Function Declarations
//
////////////////////////////////////////////////////////////////////////
unsigned long SMPP_GetCommandLength(unsigned char *buf);
unsigned long SMPP_GetCommandID(unsigned char *buf);
unsigned long SMPP_GetCommandStatus(unsigned char *buf);
unsigned long SMPP_GetSeqNo(unsigned char *buf);
unsigned short SMPP_GetOptParamID(unsigned char *buf);
unsigned short SMPP_GetOptParamLength(unsigned char *buf);
int SMPP_read(int fd, unsigned char *buf, size_t max_len);
int SMPP_send(int fd, unsigned char *buf);


////////////////////////////////////////////////////////////////////////
//
//  SMPP Command ID (from 0x00000001 to 0xFFFFFFFF)
//
////////////////////////////////////////////////////////////////////////
#define SMPP_BIND_RECEIVER		0x00000001
#define SMPP_BIND_TRANSMITTER		0x00000002
#define SMPP_QUERY_SM			0x00000003
#define SMPP_SUBMIT_SM			0x00000004
#define SMPP_DELIVER_SM			0x00000005
#define SMPP_UNBIND			0x00000006
#define SMPP_REPLACE_SM			0x00000007
#define SMPP_CANCEL_SM			0x00000008
#define SMPP_BIND_TRANSCEIVER		0x00000009
#define SMPP_OUTBIND			0x0000000B
#define SMPP_ENQUIRE_LINK		0x00000015
#define SMPP_SUBMIT_MULTI		0x00000021
#define SMPP_ALERT_NOTIFICATION		0x00000102
#define SMPP_DATA_SM			0x00000103

#define SMPP_RESPONSE_BASE		0x80000000

#define SMPP_GENERIC_NACK		SMPP_RESPONSE_BASE + 0x00000000
#define SMPP_BIND_RECEIVER_RESP		SMPP_RESPONSE_BASE + SMPP_BIND_RECEIVER
#define SMPP_BIND_TRANSMITTER_RESP	SMPP_RESPONSE_BASE + SMPP_BIND_TRANSMITTER
#define SMPP_QUERY_SM_RESP		SMPP_RESPONSE_BASE + SMPP_QUERY_SM
#define SMPP_SUBMIT_SM_RESP		SMPP_RESPONSE_BASE + SMPP_SUBMIT_SM
#define SMPP_DELIVER_SM_RESP		SMPP_RESPONSE_BASE + SMPP_DELIVER_SM
#define SMPP_UNBIND_RESP		SMPP_RESPONSE_BASE + SMPP_UNBIND
#define SMPP_REPLACE_SM_RESP		SMPP_RESPONSE_BASE + SMPP_REPLACE_SM
#define SMPP_CANCEL_SM_RESP		SMPP_RESPONSE_BASE + SMPP_CANCEL_SM
#define SMPP_BIND_TRANSCEIVER_RESP	SMPP_RESPONSE_BASE + SMPP_BIND_TRANSCEIVER
#define SMPP_SUBMIT_MULTI_RESP		SMPP_RESPONSE_BASE + SMPP_SUBMIT_MULTI
#define SMPP_ENQUIRE_LINK_RESP		SMPP_RESPONSE_BASE + SMPP_ENQUIRE_LINK
#define SMPP_DATA_SM_RESP		SMPP_RESPONSE_BASE + SMPP_DATA_SM

#define SMPP_VENDOR_RESERVED_BASE	0x00010200
#define SMPP_VENDOR_RESERVED_RESP_BASE	SMPP_RESPONSE_BASE + SMPP_VENDOR_RESERVED_BASE
#define SMPP_VENDOR_RESERVED_RANGE	0x000000FF


////////////////////////////////////////////////////////////////////////
//
//  SMPP Command Status (from 0x00000000 to 0xFFFFFFFF)
//
////////////////////////////////////////////////////////////////////////
#define ESME_ROK			0x00000000	// No Error
#define ESME_RINVMSGLEN			0x00000001	// Message Length is invalid
#define ESME_RINVCMDLEN			0x00000002	// Command Length is invalid
#define ESME_RINVCMDID			0x00000003	// Invalid Command ID
#define ESME_RINVBNDSTS			0x00000004	// Incorrect BIND Status for given command
#define ESME_RALYBND			0x00000005	// ESME Already in Bound State
#define ESME_RINVPRTFLG			0x00000006	// Invalid Priority Flag
#define ESME_RINVREGDLVFLG		0x00000007	// Invalid Registered Delivery Flag
#define ESME_RSYSERR			0x00000008	// System Error
#define ESME_RINVSRCADR			0x0000000A	// Invalid Source Address
#define ESME_RINVDSTADR			0x0000000B	// Invalid Destination Address
#define ESME_RINVMSGID			0x0000000C	// Invalid Message ID
#define ESME_RBINDFAIL			0x0000000D	// Bind Failed
#define ESME_RINVPASWD			0x0000000E	// Invalid Password
#define ESME_RINVSYSID			0x0000000F	// Invalid System ID
#define ESME_RCANCELFAIL		0x00000011	// Cancel SM Failed
#define ESME_RREPLACEFAIL		0x00000013	// Replace SM Failed
#define ESME_RMSGQFUL			0x00000014	// Message Queue Full
#define ESME_RINVSERTYP			0x00000015	// Invalid Service Type
#define ESME_RINVNUMDESTS		0x00000033	// Invalid number of destinations
#define ESME_RINVDLNAME			0x00000034	// Invalid Distribution List name
#define ESME_RINVDESTFLAG		0x00000040	// Destination Flag is invalid (submit_multi)
#define ESME_RINVSUBREP			0x00000042	// Invalid 'submit with replace' request
#define ESME_RINVESMCLASS		0x00000043	// Invalid esm_class field data
#define ESME_RCNTSUBDL			0x00000044	// Cannot Submit to Distribution List
#define ESME_RSUBMITFAIL		0x00000045	// submit_sm or submit_multi failed
#define ESME_RINVSRCTON			0x00000048	// Invalid Source Address TON
#define ESME_RINVSRCNPI			0x00000049	// Invalid Source Address NPI
#define ESME_RINVDSTTON			0x00000050	// Invalid Destination Address TON
#define ESME_RINVDSTNPI			0x00000051	// Invalid Destination Address NPI
#define ESME_RINVSYSTYP			0x00000053	// Invalid system_type field
#define ESME_RINVREPFLAG		0x00000054	// Invalid replace_if_present flag
#define ESME_RINVNUMMSGS		0x00000055	// Invalid number of messages
#define ESME_RTHROTTLED			0x00000058	// Throttling error (ESME has exceeded allowed message limits)
#define ESME_RINVSCHED			0x00000061	// Invalid Scheduled Delivery Time
#define ESME_RINVEXPIRY			0x00000062	// Invalid message validity period (Expiry time)
#define ESME_RINVDFTMSGID		0x00000063	// Predefined Message Invalid or Not Found
#define ESME_RX_T_APPN			0x00000064	// ESME Receiver Temporary App Error Code
#define ESME_RX_P_APPN			0x00000065	// ESME Receiver Permanent App Error Code
#define ESME_RX_R_APPN			0x00000066	// ESME Receiver Reject Message Error Code
#define ESME_RQUERYFAIL			0x00000067	// query_sm request failed
#define ESME_RINVOPTPARSTREAM		0x000000C0	// Error in the optional part of the PDU Body
#define ESME_ROPTPARNOTALLWD		0x000000C1	// Optional Parameter not allowed
#define ESME_RINVPARLEN			0x000000C2	// Invalid Parameter Length
#define ESME_RMISSINGOPTPARAM		0x000000C3	// Expected Optional Parameter missing
#define ESME_RINVOPTPARAMVAL		0x000000C4	// Invalid Optional Parameter Value
#define ESME_RDELIVERYFAILURE		0x000000FE	// Delivery Failure (used for data_sm_resp)
#define ESME_RUNKNOWNERR		0x000000FF	// Unknown Error

#define ESME_VENDOR_RESERVED_BASE	0x00000400	// Reserved for SMSC vendor specific errors
#define ESME_VENDOR_RESERVED_RANGE	0x000000FF


////////////////////////////////////////////////////////////////////////
//
//  SMPP Sequence Number
//
////////////////////////////////////////////////////////////////////////
#define SMPP_SEQ_MIN			0x00000001
#define SMPP_SEQ_MAX			0x7FFFFFFF


////////////////////////////////////////////////////////////////////////
//
//  SMPP Optional Parameter Tag (from 0x0001 to 0xFFFF)
//
////////////////////////////////////////////////////////////////////////
#define SMPP_OP_DEST_ADDR_SUBMIT		0x0005
#define SMPP_OP_DEST_NETWORK_TYPE		0x0006
#define SMPP_OP_DEST_BEARER_TYPE		0x0007
#define SMPP_OP_DEST_TELEMATICS_ID		0x0008
#define SMPP_OP_SOURCE_ADDR_SUBMIT		0x000D
#define SMPP_OP_SOURCE_NETWORK_TYPE		0x000E
#define SMPP_OP_SOURCE_BEARER_TYPE		0x000F
#define SMPP_OP_SOURCE_TELEMATICS_ID		0x0010
#define SMPP_OP_QOS_TIME_TO_LIVE		0x0017
#define SMPP_OP_PAYLOAD_TYPE			0x0019
#define SMPP_OP_ADDITIONAL_STATUS_INFO_TEXT	0x001D
#define SMPP_OP_RECEIPTED_MESSAGE_ID		0x001E
#define SMPP_OP_MS_MSG_WAIT_FACILITIES		0x0030
#define SMPP_OP_PRIVACY_INDICATOR		0x0201
#define SMPP_OP_SOURCE_SUBADDRESS		0x0202
#define SMPP_OP_DEST_SUBADDRESS			0x0203
#define SMPP_OP_USER_MESSAGE_REFERENCE		0x0204
#define SMPP_OP_USER_RESPONSE_CODE		0x0205
#define SMPP_OP_SOURCE_PORT			0x020A
#define SMPP_OP_DESTINATION_PORT		0x020B
#define SMPP_OP_SAR_MSG_REF_NUM			0x020C
#define SMPP_OP_LANGUAGE_INDICATOR		0x020D
#define SMPP_OP_SAR_TOTAL_SEGMENTS		0x020E
#define SMPP_OP_SAR_SEGMENT_SEQNUM		0x020F
#define SMPP_OP_SC_INTERFACE_VERSION		0x0210
#define SMPP_OP_CALLBACK_NUM_PRES_IND		0x0302
#define SMPP_OP_CALLBACK_NUM_ATAG		0x0303
#define SMPP_OP_NUMBER_OF_MESSAGES		0x0304
#define SMPP_OP_CALLBACK_NUM			0x0381
#define SMPP_OP_DBF_RESULT			0x0420
#define SMPP_OP_SET_DBF				0x0421
#define SMPP_OP_MS_AVAILABILITY_STATUS		0x0422
#define SMPP_OP_NETWORK_ERROR_CODE		0x0423
#define SMPP_OP_MESSAGE_PAYLOAD			0x0424
#define SMPP_OP_DELIVERY_FAILURE_REASON		0x0425
#define SMPP_OP_MORE_MESSAGES_TO_SEND		0x0426
#define SMPP_OP_MESSAGE_STATE			0x0427
#define SMPP_OP_USSD_SERVICE_OP			0x0501
#define SMPP_OP_DISPLAY_TIME			0x1201
#define SMPP_OP_SMS_SIGNAL			0x1203
#define SMPP_OP_MS_VALIDITY			0x1204
#define SMPP_OP_ALERT_ON_MESSAGE_DELIVERY	0x130C
#define SMPP_OP_ITS_REPLY_TYPE			0x1380
#define SMPP_OP_ITS_SESSION_INFO		0x1383

// user defined optional parameters
#define SMPP_OP_IMSI				0x4001
#define SMPP_OP_VLR				0x4002
#define SMPP_OP_TIMEOUT				0x4003
#define SMPP_OP_OA_GWNO				0x4004
#define SMPP_OP_DA_GWNO				0x4005
#define SMPP_OP_HLR				0x4006
#define SMPP_OP_CARD_TYPE			0x4007
#define SMPP_OP_TIMESTAMP			0x4008
#define SMPP_OP_TID				0x4009


////////////////////////////////////////////////////////////////////////
//
//  SMPP Delivery Receipt Format (Status Report) - Final Message States 
//
////////////////////////////////////////////////////////////////////////
#define SMPP_DRF_DELIVERED			"DELIVRD"
#define SMPP_DRF_EXPIRED			"EXPIRED"
#define SMPP_DRF_DELETED			"DELETED"
#define SMPP_DRF_UNDELIVERABLE			"UNDELIV"
#define SMPP_DRF_ACCEPTED			"ACCEPTD"
#define SMPP_DRF_UNKNOWN			"UNKNOWN"
#define SMPP_DRF_REJECTED			"REJECTD"


////////////////////////////////////////////////////////////////////////
//
//  SMPP Delivery Receipt Format Type  
//
////////////////////////////////////////////////////////////////////////
#define SMPP_DRFT_USERDEF			0
#define SMPP_DRFT_NORMAL			1

#endif		/* SMPP_H */
