#include "util.h"

/* error_monitor2
Error rate monitor (Type 2)

Input:
id:	Error counter id
err_flag:	0 = no error; otherwise = error
window: time window for error monitoring

Return:
0 = Error is accepted
2 = Error is unaccepted
3 = Error is unaccepted before
-1 = Invalid parameter
*/
int error_monitor2(celcom_alarmstruct *alarm_data, ushort id, int err_flag, time_t window)
{
	time_t	current_time = time(NULL);

	if (id >= MAX_ALARM)
		return -1;

	if (current_time - alarm_data->event_time[id] >= window){
		alarm_data->curr_base[id] = 0;	/* clear cumulative error */
		alarm_data->event_time[id] = current_time;
	}

	if (err_flag){	/* error */
		alarm_data->err[id]++;		/* increase consecutive error */
		alarm_data->curr_base[id]++;	/* increase cumulative error */
		if (alarm_data->threshold[id] && alarm_data->err[id] >= alarm_data->threshold[id]) {
			/* consecutive errors exceed threshold */
			if (alarm_data->state[id] > 0) {
				return 3;
			} else {
				alarm_data->state[id] = 1;
				return 2;
			}
		}
		else if (alarm_data->base[id] && alarm_data->curr_base[id] >= alarm_data->base[id]){
			 /* cumulative errors exceed threshold */
			if (alarm_data->state[id] > 0){
				return 3;
			} else {
				alarm_data->state[id] = 1;
				return 2;
			}
		}
	} else {
		alarm_data->err[id] = 0;	/* clear consecutive error */
	}

	return 0;
}
