/********************************************************************

	Filename	: SMPP_M_SubmitSM.h
	Description	: SMPP SubmitSM Message class header file

	History:
	------------------------------------------------------------
	20010707	fw: First Creation
	20010808	kei: Add User Message Reference
	20010816	fw: Added Timeout Parameter
	20011123	kei: Add OP for PTC-SMX as GWNO of OA and DA
	20020109	kei: Add SMPP_P_HLR optional param

********************************************************************/

#ifndef SMPP_M_SUBMITSM_H
#define SMPP_M_SUBMITSM_H

#include "SMPP_M_Request.h"
#include "SMPP_P_ServiceType.h"
#include "SMPP_P_AddrTON.h"
#include "SMPP_P_AddrNPI.h"
#include "SMPP_P_Addr.h"
#include "SMPP_P_ESMClass.h"
#include "SMPP_P_ProtocolID.h"
#include "SMPP_P_PriorityFlag.h"
#include "SMPP_P_ScheduleDeliveryTime.h"
#include "SMPP_P_ValidityPeriod.h"
#include "SMPP_P_RegisteredDelivery.h"
#include "SMPP_P_ReplaceIfPresentFlag.h"
#include "SMPP_P_DataCoding.h"
#include "SMPP_P_SM_DefaultMsgID.h"
#include "SMPP_P_SM_Length.h"
#include "SMPP_P_ShortMessage.h"
#include "SMPP_P_UserMessageReference.h"
#include "SMPP_P_MoreMsgFlag.h"
#include "SMPP_P_UssdServiceOp.h"
#include "SMPP_P_IMSI.h"
#include "SMPP_P_VLR.h"
#include "SMPP_P_HLR.h"
#include "SMPP_P_Timeout.h"
#include "SMPP_P_DA_GWNO.h"
#include "SMPP_P_OA_GWNO.h"

class SMPP_M_SubmitSM : public SMPP_M_Request {
public:
	// mandatory parameters
	SMPP_P_ServiceType		service_type;
	SMPP_P_AddrTON			source_addr_ton;
	SMPP_P_AddrNPI			source_addr_npi;
	SMPP_P_Addr			source_addr;
	SMPP_P_AddrTON			dest_addr_ton;
	SMPP_P_AddrNPI			dest_addr_npi;
	SMPP_P_Addr			dest_addr;
	SMPP_P_ESMClass			esm_class;
	SMPP_P_ProtocolID		protocol_id;
	SMPP_P_PriorityFlag		priority_flag;
	SMPP_P_ScheduleDeliveryTime	schedule_delivery_time;
	SMPP_P_ValidityPeriod		validity_period;
	SMPP_P_RegisteredDelivery	registered_delivery;
	SMPP_P_ReplaceIfPresentFlag	replace_if_present_flag;
	SMPP_P_DataCoding		data_coding;
	SMPP_P_SM_DefaultMsgID		sm_default_msg_id;
	SMPP_P_SM_Length		sm_length;
	SMPP_P_ShortMessage		short_message;

	// optional parameters
	SMPP_P_UserMessageReference	user_message_reference;
	SMPP_P_MoreMsgFlag		more_message_to_send;
	SMPP_P_UssdServiceOp		ussd_service_op;
	SMPP_P_IMSI			imsi;
	SMPP_P_VLR			vlr;
	SMPP_P_HLR			hlr;
	SMPP_P_Timeout			timeout;
	SMPP_P_DA_GWNO			dest_addr_gwno;
	SMPP_P_OA_GWNO			source_addr_gwno;

	SMPP_M_SubmitSM();
	~SMPP_M_SubmitSM();

	size_t				pack_msg(unsigned char *buf);
	size_t				unpack_msg(unsigned char *buf);
};

#endif		/* SMPP_M_SUBMITSM_H */
