/********************************************************************

	Filename      : smppClient.c
	Description   : SMPP Client C++ program 
	Version       : 1.00

	History		
	------------------------------------------------------------

********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <thread.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <signal.h>
#include <ctype.h>
#include <fcntl.h>

#include <IPC_Memory.h>
#include <GeneralErrorException.h>
#include <LogUtil.h>
#include <SysUtil.h>
#include <INTM_Message.h>
#include <intm_msg.h>
#include <SMPP_incl.h>  

#include "intm_shared.h"
#include "smppClient.h"
#include "log.h"

#define MODULE_NAME	"SMPPCLNT"
#define VERSION_NUM	"1.0"
#define REVISION_DATE	"20020801"
static char version[]="\n" MODULE_NAME " Version " VERSION_NUM " Copyright (c) IVRS "REVISION_DATE "\n";

#define M_INIT(x) ( memset(x, 0, sizeof(x)) )

smppClient clnt;
unsigned char logbuf[1024];
int process_id, mtype = 0;

extern "C" {
        void signal_hndlr(int sig_num);
};

void signal_hndlr(int sig_num)
{
	switch (sig_num) {
	case SIGSEGV:
	case SIGBUS:
	case SIGTERM:
	case SIGQUIT:
		printf(MODULE_NAME" exit. received signal no. = %d\n", sig_num);
		printf("disconnectServer()\n");
		clnt.disconnectServer(0);
		clnt.quit = 1;   
		break;

	case SIGCHLD:
        case SIGPIPE:
	case SIGINT:
	case SIGUSR1:
	case SIGUSR2:
	case SIGALRM:
		signal(sig_num, signal_hndlr);
		break;

	default:
		break;
	}

}

int main(int argc, char **argv)
{

	if (argc < 7) {
		printf("Usage:\n");
		printf("	%s <host> <server port> <client port> <user> <password> <process_id>\n", argv[0]);
		return -1;
	}

	init_log((char*)MODULE_NAME);

        loginfo(PROCESS_LOG,
                INFO,
                (char *)"smppClient starting...");

	try {
		signal(SIGINT, signal_hndlr);
		signal(SIGQUIT, signal_hndlr);
		signal(SIGTERM, signal_hndlr);
		signal(SIGALRM, signal_hndlr);
		signal(SIGSEGV, signal_hndlr);
		signal(SIGUSR1, signal_hndlr);
		signal(SIGUSR2, signal_hndlr);
		signal(SIGPIPE, signal_hndlr);
                signal(SIGCHLD, signal_hndlr);
		msleep(200);
		unsigned short local_port = (argc < 4) ? 0 : atoi(argv[3]);

		strcpy((char*)clnt.user, argv[4]);
		strcpy((char*)clnt.password, argv[5]);
		process_id = atoi(argv[6]);
		mtype = 9000+process_id;

		if (clnt.setServerConfig(0, argv[1], (unsigned short)atoi(argv[2]), local_port) < 0) {
			throw GeneralErrorException((char*)"invalid server ip and port");
		}

		int rc = clnt.connectServer(0);  // connect server 
		printf("connect server rc=%d\n", rc);
		msleep(200);

		try {
			clnt.my_q.init(USSDCQ_ID, IPC_CREAT| PERMS);    // smppClient Q
			clnt.dispatch_q.init(UDISPQ_ID, PERMS);  	// smppDispatch Q
			clnt.init();	/**INIT SHARED MEM**/
			clnt.waitEvent();    
		} catch (GeneralErrorException err) {
			printf("%s\n", err.getMessage());
			return 0;
		}

		clnt.disconnectServer(0);

	} catch (GeneralErrorException err) {
		printf("Catched GeneralErrorException %s\n", err.getMessage());
	}
	return 0;
}


int smppClient::init()
{

	return 1;
}


smppClient::smppClient()
{
	login_flag = 0;
	quit = 0;
	memset(user, 0, sizeof(user));
	memset(password, 0, sizeof(password));
	enquirelink_count = 0;
	enquirelink_flag = 0;
	last_activity_time = time(NULL);
}

smppClient::~smppClient()
{
}

smppClient& smppClient::waitEvent()
{
	int		i, rc;
	unsigned char 	in_msg[65536];
	size_t 		in_msg_size;
	time_t		current_time = time(NULL);
	
	while (!quit) {

		current_time = time(NULL);

		if (getNumActiveServers() <= 0) {

        		loginfo(PROCESS_LOG,
            		        MAJOR,
                               (char *)"Connection failed, no active server");

			printf("no active server, sleep 3s\n");
			sleep(3);

			int rc = connectServer(0);  // connect server 
			printf("connect server rc=%d\n", rc);

                        if (rc == 0){
        			loginfo(PROCESS_LOG,
            		        	INFO,
                               		(char *)"Connection successful");

				login_flag = 0;
			}

			enquirelink_flag = 0;
                        enquirelink_count = 0;
                        last_activity_time = time(NULL);

		} else {
			if ( login_flag == 0 ) {  	// login to to Gateway 
				printf("Loging into USSD Gateway \n");
				login_flag=1;           // login in progress
				loginServer(0, 3); 	// Transceiver - exit if login failed

			} else if ( login_flag == 1) {
				printf("Login in progress\n");
				sleep(1);
			} else if ( login_flag == 2) { 	// login successfully

				M_INIT(in_msg);
				rc=my_q.recv_nowait(in_msg, in_msg_size, mtype);  
 

				if (rc > 0) {
				printf("Msg Length [%d]\n",in_msg_size);
					processIntmMessage((intm_msg*) in_msg, in_msg_size); 

				}
			}

			if (enquirelink_flag == 1){
                        	if (current_time - last_activity_time >= 300){
                                	check_enquireLink_timeout();
                        	}
			}

                	if (current_time - last_activity_time >= 60){
				enquirelink_flag = 1;
                      		enquireLink(0);
                        	last_activity_time = time(NULL);
                	}

			// poll message from Gateway
			pollServerEvent(20);		// timeout = 500 milliseconds
		}
	}	// end of while (!quit)

	clnt.disconnectServer(0);

	// logout and close socket

	return *this;
}

void smppClient::processIntmMessage(intm_msg* intm, int in_msg_size)
{
	SMPP_M_SubmitSM		smpp_s;
	unsigned char           utype[10];
	unsigned char           more_msg[10];
	unsigned char 		msisdn[30];
	unsigned char 		imsi[30];
	unsigned char 		ussd_data[500];
	unsigned char		uc_out_smpp[2000];
	int			param_len,msg_length;
        unsigned char           dcs = 0;

	memset(utype, 0, sizeof(utype));
	intm->get_param(UD_UTYPE, utype, &param_len);
	utype[param_len]=0;
	printf("UTYPE = %s \n", utype);

	memset(msisdn, 0, sizeof(msisdn));
	intm->get_param(ORIG_NUM, msisdn, &param_len);
	msisdn[param_len]=0;
	printf("MSISDN = %s\n", msisdn);

	memset(imsi, 0, sizeof(imsi));
	intm->get_param(SERIAL_NUM, imsi, &param_len);
	imsi[param_len]=0;
	printf("IMSI = %s\n", imsi);

	memset(ussd_data, 0, sizeof(ussd_data));
	intm->get_param(UD_SHORT_MSG, ussd_data, &msg_length);
	ussd_data[msg_length]=0;
	printf("SHORT MSG = [%d]\n",msg_length);
        for (int j=0;j<msg_length;j++)
	printf("%02X ",ussd_data[j]);
	printf("\n");
	
	memset(more_msg, 0, sizeof(more_msg));
	intm->get_param(UD_MORE_MSG_TO_SEND, more_msg, &param_len);
	more_msg[param_len]=0;
	printf("MORE MSG = %s\n", more_msg);

	smpp_s.ussd_service_op.set_value(atoi((char*)utype)); 	// PUSSR response phase 2 
	smpp_s.dest_addr.set_string((char*)msisdn);
	smpp_s.imsi.set_content(imsi,strlen((char*)imsi));
	smpp_s.sm_length.set_value(msg_length);
	smpp_s.short_message.set_content(ussd_data, msg_length);

	intm->get_param(UD_ACTCODE, (unsigned char*)&dcs, &param_len);
	printf("DCS = %d, param_len = %d\n", dcs,param_len);

        if (param_len == 0)
          {
           dcs=15;
           dcs %= 255;
          }

	printf("DCS = %d\n", dcs);

       	loginfo(PROCESS_LOG,
       		INFO,
       		(char *)"Recv from EA, MSISDN[%s], DCS[%d], USSD length[%d]", msisdn,dcs,msg_length);

	smpp_s.data_coding.set_value(dcs);

	smpp_s.more_message_to_send.set_value(atoi((char*)more_msg));  // end message

	memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
	smpp_s.pack_msg(uc_out_smpp);


	sendServerSMPPMessage(0, uc_out_smpp);  // send to default socket /
}

int smppClient::processServerEvent(int id)
{
	// read message from server

	unsigned char		uc_in_smpp[2000];
        char                    tmpbuf[256];

	int len = readServerSMPPMessage(id, uc_in_smpp, sizeof(uc_in_smpp)-1);
	if (len <= 0) {
		return -1;
	}

	// process server message
	unsigned char		uc_out_smpp[2000];
	unsigned char		parameter[500];
	unsigned char 		msisdn[50], imsi[50], vlr[50],hlr[50], ussd_string[200];
	int 			utype=0, status=-1;
	
	try {
		switch (SMPP_GetCommandID(uc_in_smpp)) {
		case SMPP_BIND_RECEIVER_RESP:
		case SMPP_BIND_TRANSMITTER_RESP:
		case SMPP_BIND_TRANSCEIVER_RESP: {
			SMPP_M_BindResp smpp_r_b;
			status=-1;

			printf("Bind Response Received, Message Type = 0x%08X\n", SMPP_GetCommandID(uc_in_smpp));
			smpp_r_b.unpack_msg(uc_in_smpp);
			status = smpp_r_b.header.command_status.get_value();
			printf("Status = %08X\n", status );

			if ( status == 0 ) {
				login_flag = 2;

        			loginfo(PROCESS_LOG,
            	        		INFO,
                      			(char *)"Bind response success");

			} else { 	// login failed, log error msg and restart [loginfo]
				login_flag = 0;
				enquirelink_count = 0;
                		last_activity_time = time(NULL);

				disconnectServer(0);

				printf("Login failed, disconnectServer()\n");

        			loginfo(PROCESS_LOG,
            	        		MAJOR,
                      			(char *)"Login failed, server disconnected");
			}
			
			break;
		}
		case SMPP_UNBIND_RESP: {
			SMPP_M_UnbindResp smpp_r_ub;
			printf("Unbind Response Received\n");
			smpp_r_ub.unpack_msg(uc_in_smpp);
			status = smpp_r_ub.header.command_status.get_value();
			printf("Status = %08X\n", status);
			// log response and log error message if failed. [loginfo]
			break;
		}
		case SMPP_SUBMIT_SM_RESP: {
			SMPP_M_SubmitSMResp smpp_r_s;
			printf("Submit SM Response Received\n");
			smpp_r_s.unpack_msg(uc_in_smpp);
			status = smpp_r_s.header.command_status.get_value();
			printf("Status = %08X\n", status);
			// log response and log error message if failed. [loginfo]
			break;
		}
		case SMPP_DELIVER_SM: {
			SMPP_M_DeliverSM smpp_d;
			SMPP_M_DeliverSMResp smpp_r_d;

			// This 2 line for TESTING
			smpp_d.imsi.set_content(imsi, strlen((char*)imsi));
			smpp_d.hlr.set_content(hlr, strlen((char*)hlr));
			
			printf("Deliver SM Received\n");
			smpp_d.unpack_msg(uc_in_smpp, SMPP_DRFT_USERDEF);
			if (smpp_d.ussd_service_op.is_present()) {
				utype = smpp_d.ussd_service_op.get_value();
				printf("USSD Type = %d\n", utype );
			}

			if (smpp_d.imsi.is_present()) {
				memset(imsi, 0, sizeof(imsi));
				smpp_d.imsi.get_content(imsi);
				printf("IMSI = %s\n", imsi);
			}

			if (smpp_d.vlr.is_present()) {
				memset(vlr, 0, sizeof(vlr));
				smpp_d.vlr.get_content(vlr);
				printf("VLR = %s\n", vlr);
			}

			if (smpp_d.hlr.is_present()) {
				memset(hlr, 0, sizeof(hlr));
				smpp_d.hlr.get_content(hlr);
				printf("HLR = %s\n", hlr);
			}

			memset(parameter, 0, sizeof(parameter));
			smpp_d.source_addr.get_content(parameter);
			memset(msisdn, 0, sizeof(msisdn));
			strcpy((char*)msisdn, (char*)parameter);
			printf("MSISDN = %s\n", msisdn );

			memset(parameter, 0, sizeof(parameter));
			smpp_d.short_message.get_content(parameter);
			memset(ussd_string, 0, sizeof(ussd_string));
			strcpy((char*)ussd_string, (char*)parameter);
			printf("USSD Content = %s\n", ussd_string);

			// send DELIVERY_SM response
			smpp_r_d.header.seq_num.set_value(smpp_d.header.seq_num.get_value());
			memset(uc_out_smpp, 0, sizeof(uc_out_smpp));
			smpp_r_d.pack_msg(uc_out_smpp);
			sendServerSMPPMessage(id, uc_out_smpp);
			
			memset(tmpbuf, 0, sizeof(tmpbuf));
			smpp_d.source_addr.get_string(tmpbuf);
			printf("SourceAddr [%s]\n", tmpbuf);
			printf("TypeOfNumberDestAddr [%d]\n",  smpp_d.dest_addr_ton.get_value());
			printf("NumberPlanIndicatorDestAddr [%d]\n", smpp_d.dest_addr_npi.get_value());

			memset(tmpbuf, 0, sizeof(tmpbuf));
			smpp_d.dest_addr.get_string(tmpbuf);
			printf("DestAddr [%s]\n", tmpbuf);
			printf("ESMClass [%d]\n", smpp_d.esm_class.get_value());
			printf("ProtocolID [%d]\n", smpp_d.protocol_id.get_value());
			printf("PriorityFlag [%d]\n", smpp_d.priority_flag.get_value());

			memset(tmpbuf, 0, sizeof(tmpbuf));
			smpp_d.schedule_delivery_time.get_string(tmpbuf);
			printf("ScheduleDeliveryTime [%s]\n", tmpbuf);

			memset(tmpbuf, 0, sizeof(tmpbuf));
			smpp_d.validity_period.get_string(tmpbuf);
			printf("ValidityPeriod [%s]\n", tmpbuf);
			printf("RegisteredDelivery [%d]\n", smpp_d.registered_delivery.get_value());
			printf("ReplaceIfPresentFlag [%d]\n", smpp_d.replace_if_present_flag.get_value());
			printf("DataCoding [%d]\n", smpp_d.data_coding.get_value());
			printf("SMDefaultMsgID [%d]\n", smpp_d.sm_default_msg_id.get_value());
			printf("SMLength [%d]\n", smpp_d.sm_length.get_value());

			unsigned char sm[500];
			memset(sm, 0, sizeof(sm));
			smpp_d.short_message.unpack_param(sm);
			printf("ShortMessage [%s]\n", sm);
/****/

			packEAIntmMessage(utype, msisdn, ussd_string, imsi, vlr,hlr);

			enquirelink_flag = 0;
                        enquirelink_count = 0;
                        last_activity_time = time(NULL);

			break;
		}
		case SMPP_ENQUIRE_LINK: {
                        SMPP_M_EnquireLink smpp;
                        SMPP_M_EnquireLinkResp smpp_resp;

                        smpp.unpack_msg(uc_in_smpp);
                        printf("smppClient <-- EA_Handler (receive Enquire Link request from USSDGW)\n");
                        smpp_resp.header.seq_num.set_value(smpp.header.seq_num.get_value());
                        unsigned char uc_smpp[1000];
                        memset(uc_smpp, 0, sizeof(uc_smpp));
                        smpp_resp.pack_msg(uc_smpp);
                        sendServerSMPPMessage(id, uc_smpp);
                        printf("smppClient --> EA_Handler (responding Enquire Link to USSDGW)\n");

			enquirelink_flag = 0;
                        enquirelink_count = 0;
                        last_activity_time = time(NULL);

                        break;
                }
                case SMPP_ENQUIRE_LINK_RESP: {
                        SMPP_M_EnquireLinkResp smpp_r_el;
                        smpp_r_el.unpack_msg(uc_in_smpp);
                        status = smpp_r_el.header.command_status.get_value();
                        printf("smppClient <-- EA_Handler (received Enquire Link response from USSDGW, status=[%08X])\n", status);

                        enquirelink_flag = 0;
                        enquirelink_count = 0;
			last_activity_time = time(NULL);

                        break;
		}
		default:
			printf("Default Message Received, Message Type = 0x%08X\n", SMPP_GetCommandID(uc_in_smpp));

        		loginfo(PROCESS_LOG,
            	       		INFO,
                      		(char *)"Message Received, Message Type = 0x%08X",SMPP_GetCommandID(uc_in_smpp));

			enquirelink_flag = 0;
                        enquirelink_count = 0;
                        last_activity_time = time(NULL);

			break;
		}
	} catch (SMPP_Exception smpp_err) {
		printf("SMPP Exception Catched!!!\n");
	}

	printf("--------------------------------------------------\n");

	return 0;
}

int smppClient::getNextSeqNum()
{
	seq_num++;
	if (seq_num < 0x00000000 || seq_num > 0x7fffffff) {
		seq_num = 0;
	}
	return seq_num;
}

void smppClient::loginServer(int id, int logintype)
{
	SMPP_M_Bind		smpp;

	printf("user: %s password: %s\n", user, password);  // temporary

	smpp.header.command_id.set_value(SMPP_BIND_TRANSCEIVER);
	smpp.header.seq_num.set_value(getNextSeqNum());
	smpp.system_id.set_string((char*)user);
	smpp.password.set_string((char*)password);

	unsigned char		uc_smpp[2000];
	memset(uc_smpp, 0, sizeof(uc_smpp));
	smpp.pack_msg(uc_smpp);

	sendServerSMPPMessage(id, uc_smpp);

	return;
}

int smppClient::connectServer(int id)
{
        int rc = 0;
        int fd=0, flags=0;

        rc = SMPP_TCPClient::connectServer(id);
        if (rc == 0) {
                fd = tcpsock[id].getsockfd();
                flags = fcntl(fd, F_GETFL, 0);;

                if ((flags & O_NONBLOCK) == 0) {
                        flags |= O_NONBLOCK;
                        fcntl(fd, F_SETFL, flags);
                }
        } else {
        }
        return rc;
}


int smppClient::disconnectServer(int id)
{
        int rc = 0;

       	loginfo(PROCESS_LOG,
       		MAJOR,
       		(char *)"TCP connection break");

        rc = SMPP_TCPClient::disconnectServer(id);
        return rc;
}

void smppClient::logoutServer(int id)
{
	SMPP_M_Unbind		smpp_ub;
	smpp_ub.header.seq_num.set_value(getNextSeqNum());

	unsigned char		uc_smpp[2000];
	memset(uc_smpp, 0, sizeof(uc_smpp));
	smpp_ub.pack_msg(uc_smpp);
	sendServerSMPPMessage(id, uc_smpp);

	return;
}

void smppClient::packEAIntmMessage(int utype, unsigned char *msisdn, unsigned char* ussd_string, unsigned char* imsi, unsigned char* vlr,unsigned char* hlr)
{
	intm_msg	out_msg;
	int 		out_msg_size;
	unsigned char   utype_s[3];

	memset(utype_s, 0, sizeof(utype_s));
	sprintf((char*)utype_s, "%d", utype);
	printf("utype_s [%s], DATA [%d]\n",(char*)utype_s,DATA);

	out_msg.init_msg(USSDC, UDISP, 1, 1, 0, DATA);

	out_msg.add_param(UD_UTYPE, utype_s, strlen((char*)utype_s));		// utype
	out_msg.add_param(ORIG_NUM, msisdn, strlen((char*)msisdn));	// MSISDN
	out_msg.add_param(UD_USSD_STRING, ussd_string, strlen((char*)ussd_string));	 // USSD string
	out_msg.add_param(SERIAL_NUM, imsi, strlen((char*)imsi));

	if ( strlen((char*)vlr) > 0 ) {
		out_msg.add_param(UD_VLR, vlr, strlen((char*)vlr));
	}

	if ( strlen((char*)hlr) > 0 ) {
		out_msg.add_param(UD_HLR, hlr, strlen((char*)hlr));
	}

	printf("Send to smppDispatch, mtype %d\n",mtype);
	dispatch_q.send(&out_msg, out_msg.len_indicator+2, mtype, 3);

       	loginfo(PROCESS_LOG,
       		INFO,
       		(char *)"Send to EA, MSISDN[%s], USSD String[%s]", msisdn,ussd_string);

}

void smppClient::check_enquireLink_timeout()
{
        if(enquirelink_count >= 3){
                printf("No response timeout for EnquireLink reached threshold, disconnect server\n");

       		loginfo(PROCESS_LOG,
       			MAJOR,
       			(char *)"No response timeout for EnquireLink reached threshold, disconnect server");

		login_flag = 0;
		disconnectServer(0);

		int rc = connectServer(0);  // connect server
                printf("connect server rc=%d\n", rc);

                if (rc == 0){
                     login_flag = 0;
                }

		enquirelink_count = 0;
		enquirelink_flag = 0;
                last_activity_time = time(NULL);

        }else{
       		loginfo(PROCESS_LOG,
       			MAJOR,
       			(char *)"No response timeout for EnquireLink, reset timer");

                printf("No response timeout for EnquireLink, reset timer\n");
                last_activity_time = time(NULL);
                enquirelink_count++;
        }

        return;
}

void smppClient::enquireLink(int id)
{
        SMPP_M_EnquireLink      smpp_el;

        printf("smppClient --> EA_Handler (sending Enquire Link request to USSDGW)\n");

        smpp_el.header.command_id.set_value(SMPP_ENQUIRE_LINK);
        smpp_el.header.seq_num.set_value(getNextSeqNum());

        unsigned char           uc_smpp[2000];
        memset(uc_smpp, 0, sizeof(uc_smpp));
        smpp_el.pack_msg(uc_smpp);
        sendServerSMPPMessage(id, uc_smpp);

        return;
}
